@@ -1,5 +1,5 @@
 #!/bin/sh
-VERSION=`cat xlock/version.h | cut -d\" -f2 | cut -d\- -f2`
+VERSION=`cat xlock/version.h | cut -d\" -f2`
 NAME=xlockmore
 PROG=$NAME-$VERSION
 AUTHOR="David Bagley"
@@ -12,10 +12,11 @@ if test "$VERSION" = ""; then
 	echo "run from $NAME main directory"
 	exit
 fi
+#make -f Makefile.win32 distclean
 make -f Makefile.in distclean
 mkdir -p $PROG
 LIST=`ls`
-EXCEPT="linux solaris $PROG"
+EXCEPT="linux solaris cygwin $PROG"
 for i in $LIST; do
 	FOUND=0
 	for j in $EXCEPT; do
@@ -32,9 +33,5 @@ tar cvf - $PROG | bzip2 > $PROGZIP
 rm -rf $PROG
 mv $PROGZIP ..
 if test "$1" != "-p"; then
-	cd ..
-	gpg --print-md SHA1 $PROGZIP > $PROGZIP.sha
-	gpg --print-md MD5 $PROGZIP > $PROGZIP.md5
-	gpg --armor --output $PROGZIP.asc --detach-sig $PROGZIP
-	gpg --verify $PROGZIP.asc $PROGZIP
+	echo "run signbz2.sh next"
 fi
