@@ -48,7 +48,7 @@ static const char sccsid[] = "@(#)dclock.c	5.00 2000/11/01 xlockmore";
  *
  *  Time 0 is 0:00:00 1 Jan 1970 at least according to hard core UNIX fans
  *  Minutes from 1  Jan 1970 to 21 Jun 1985:  8137440
- *  Minutes from 21 Jun 1985 to 12 Jul 1988:  6867360
+ *  Minutes from 21 Jun 1985 to 12 Jul 1998:  6867360
  *                                    Total: 15004800
  *
  *  Population Explosion Saver (popex)
@@ -131,6 +131,7 @@ static const char sccsid[] = "@(#)dclock.c	5.00 2000/11/01 xlockmore";
 /* epoch time at midnight 1 January 2000 UTC */
 #define Y2K_TIME_START (((30 * 365) + 7) * 24 * 60 * 60)
 #define Y2001_TIME_START (Y2K_TIME_START + 366 * 24 * 60 * 60)
+#define MAYAN_TIME_START (((42 * 365) + 10) * 24 * 60 * 60  + (366 - 11) * 24 * 60 * 60)
 
 #define LED_LEAN 0.2
 #define LED_XS 30.0
@@ -152,6 +153,7 @@ static const char sccsid[] = "@(#)dclock.c	5.00 2000/11/01 xlockmore";
 #define DEF_TIME24 "False"
 #define DEF_Y2K    "False"
 #define DEF_Y2001  "False"
+#define DEF_MAYAN  "False"
 
 /*- If you remember your Electronics course...
         a
@@ -213,6 +215,7 @@ static Bool veg;
 static Bool time24;
 static Bool y2k;
 static Bool millennium;
+static Bool mayan;
 
 static XrmOptionDescRec opts[] =
 {
@@ -235,7 +238,9 @@ static XrmOptionDescRec opts[] =
 	{(char *) "-y2k", (char *) ".dclock.y2k", XrmoptionNoArg, (caddr_t) "on"},
 	{(char *) "+y2k", (char *) ".dclock.y2k", XrmoptionNoArg, (caddr_t) "off"},
 	{(char *) "-millennium", (char *) ".dclock.millennium", XrmoptionNoArg, (caddr_t) "on"},
-	{(char *) "+millennium", (char *) ".dclock.millennium", XrmoptionNoArg, (caddr_t) "off"}
+	{(char *) "+millennium", (char *) ".dclock.millennium", XrmoptionNoArg, (caddr_t) "off"},
+	{(char *) "-mayan", (char *) ".dclock.mayan", XrmoptionNoArg, (caddr_t) "on"},
+	{(char *) "+mayan", (char *) ".dclock.mayan", XrmoptionNoArg, (caddr_t) "off"}
 };
 static argtype vars[] =
 {
@@ -248,7 +253,8 @@ static argtype vars[] =
 	{(void *) & veg, (char *) "veg", (char *) "Veg", (char *) DEF_VEG, t_Bool},
 	{(void *) & time24, (char *) "time24", (char *) "Time24", (char *) DEF_TIME24, t_Bool},
 	{(void *) & y2k, (char *) "y2k", (char *) "Y2K", (char *) DEF_Y2K, t_Bool},
-	{(void *) & millennium, (char *) "millennium", (char *) "Millennium", (char *) DEF_Y2001, t_Bool}
+	{(void *) & millennium, (char *) "millennium", (char *) "Millennium", (char *) DEF_Y2001, t_Bool},
+	{(void *) & mayan, (char *) "mayan", (char *) "Mayan", (char *) DEF_MAYAN, t_Bool}
 };
 static OptionStruct desc[] =
 {
@@ -262,6 +268,7 @@ static OptionStruct desc[] =
 	{(char *) "-/+time24", (char *) "turn on/off 24 hour display"},
 	{(char *) "-/+y2k", (char *) "turn on/off Year 2000 countdown"},
 	{(char *) "-/+millennium", (char *) "turn on/off 3rd Millennium (1 January 2001) countdown"},
+	{(char *) "-/+mayan", (char *) "turn on/off 13 bak'tun (21 December 2012) countdown"},
 };
 
 ModeSpecOpt dclock_opts =
@@ -360,7 +367,7 @@ typedef struct {
 	Pixmap      pixmap;
 	GC          fgGC, bgGC;
 	Bool        popex, forest, hiv, lab, veg;
-	Bool        time24, y2k, millennium, led, binary;
+	Bool        time24, y2k, millennium, mayan, led, binary;
 	XPoint      parallelogram[4];
 } dclockstruct;
 
@@ -368,13 +375,19 @@ static dclockstruct *dclocks = (dclockstruct *) NULL;
 
 extern char *message;
 
-static unsigned long
-timeAtLastNewYear(long timeNow)
+static time_t
+timeAtLastNewYear(time_t timeNow)
 {
 	struct tm *t;
 
-	t = localtime((const time_t *) &timeNow);
-	return (unsigned long)(t->tm_year);
+	t = localtime(&timeNow);
+	t->tm_mon = 0;
+	t->tm_mday = 1;
+	t->tm_hour = 0;
+	t->tm_min = 0;
+	t->tm_sec = 0;
+	t->tm_isdst = -1; /* mktime shall decide itself */
+	return mktime(t);
 }
 
 #ifndef HAVE_SNPRINTF
@@ -413,7 +426,7 @@ convert(double x, char *string)
 }
 
 static void
-dayhrminsec(long timeCount, int tzoffset, char *string)
+dayhrminsec(time_t timeCount, int tzoffset, char *string)
 {
 	int days, hours, minutes, secs;
 	int bufsize, i;
@@ -650,13 +663,13 @@ drawDclock(ModeInfo * mi)
 		dp->timeold = dp->timenew = time((time_t *) NULL);
 		dp->str = ctime(&dp->timeold);
 	  } else if (!dp->popex && !dp->forest && !dp->hiv &&
-	    !dp->lab && !dp->veg && !dp->y2k && !dp->millennium) {
+	    !dp->lab && !dp->veg && !dp->y2k && !dp->millennium && !dp->mayan) {
 		if (dp->timeold != (dp->timenew = time((time_t *) NULL))) {
 			/* only parse if time has changed */
 			dp->timeold = dp->timenew;
 
 			if (!dp->popex && !dp->forest && !dp->hiv && !dp->lab &&
-			    !dp->veg && !dp->y2k && !dp->millennium) {
+			    !dp->veg && !dp->y2k && !dp->millennium && !dp->mayan) {
 			  if (dp->time24)
 				(void) strftime(dp->strpta[0], STRSIZE,
 					"%H:%M:%S", localtime(&(dp->timeold)));
@@ -668,9 +681,9 @@ drawDclock(ModeInfo * mi)
 				"%a %b %d %Y", localtime(&(dp->timeold)));
 		}
 	  } else {
-		long timeNow, timeLocal;
+		time_t timeNow, timeLocal;
 		timeNow = seconds();
-		timeLocal = timeNow - dp->tzoffset;
+		timeLocal = timeNow + dp->tzoffset;
 
 		if (dp->popex) {
 			convert(PEOPLE_TIME_START + (PEOPLE_MIN / 60.0) * timeNow, dp->strnew[1]);
@@ -722,7 +735,18 @@ drawDclock(ModeInfo * mi)
 			else
 				dp->strpta[0] = (char *) POST_Y2001_STRING;
 			dp->strptb[0] = dp->strpta[0];
-			dayhrminsec(Y2001_TIME_START - timeLocal, dp->tzoffset, dp->strnew[1]);
+			dayhrminsec(Y2001_TIME_START - timeLocal,
+				dp->tzoffset, dp->strnew[1]);
+			dp->strpta[1] = dp->strnew[1];
+			dp->strptb[1] = dp->strpta[1];
+		} else if (dp->mayan) {
+			if (MAYAN_TIME_START >= timeLocal)
+				dp->strpta[0] = (char *) MAYAN_STRING;
+			else
+				dp->strpta[0] = (char *) POST_MAYAN_STRING;
+			dp->strptb[0] = dp->strpta[0];
+			dayhrminsec(MAYAN_TIME_START - timeLocal,
+				dp->tzoffset, dp->strnew[1]);
 			dp->strpta[1] = dp->strnew[1];
 			dp->strptb[1] = dp->strpta[1];
 		}
@@ -932,7 +956,7 @@ init_dclock(ModeInfo * mi)
 {
 	Display *display = MI_DISPLAY(mi);
 	dclockstruct *dp;
-	long timeNow, timeLocal;
+	time_t timeNow, timeLocal;
 	int i, j;
 
 	if (dclocks == NULL) {
@@ -960,17 +984,18 @@ init_dclock(ModeInfo * mi)
 	dp->time24 = False;
 	dp->y2k = False;
 	dp->millennium = False;
+	dp->mayan = False;
 	dp->lines = 2;
-#if defined(MODE_dclock_y2k) && defined(MODE_dclock_millennium)
+#if defined(MODE_dclock_y2k) && defined(MODE_dclock_millennium) && defined(MODE_dclock_mayan)
+#define NUM_DCLOCK_MODES 12
+#endif
+#if (defined(MODE_dclock_y2k) && defined(MODE_dclock_millennium) && !defined(MODE_dclock_mayan)) || (defined(MODE_dclock_y2k) && !defined(MODE_dclock_millennium) && defined(MODE_dclock_mayan)) || (!defined(MODE_dclock_y2k) && defined(MODE_dclock_millennium) && defined(MODE_dclock_mayan))
 #define NUM_DCLOCK_MODES 11
 #endif
-#if defined(MODE_dclock_y2k) && !defined(MODE_dclock_millennium)
+#if (!defined(MODE_dclock_y2k) && !defined(MODE_dclock_millennium) && defined(MODE_dclock_mayan)) || (!defined(MODE_dclock_y2k) && defined(MODE_dclock_millennium) && !defined(MODE_dclock_mayan)) || (defined(MODE_dclock_y2k) && !defined(MODE_dclock_millennium) && !defined(MODE_dclock_mayan))
 #define NUM_DCLOCK_MODES 10
 #endif
-#if !defined(MODE_dclock_y2k) && defined(MODE_dclock_millennium)
-#define NUM_DCLOCK_MODES 10
-#endif
-#if !defined(MODE_dclock_y2k) && !defined(MODE_dclock_millennium)
+#if !defined(MODE_dclock_y2k) && !defined(MODE_dclock_millennium) && !defined(MODE_dclock_mayan)
 #define NUM_DCLOCK_MODES 9
 #endif
 	if (!(message && *message)) {
@@ -1002,18 +1027,34 @@ init_dclock(ModeInfo * mi)
 			case 8:
 				dp->binary = True;
 				break;
-#if defined(MODE_dclock_y2k) || defined(MODE_dclock_millennium)
+#if defined(MODE_dclock_y2k) || defined(MODE_dclock_millennium) || defined(MODE_dclock_mayan)
 			case 9:
 #ifdef MODE_dclock_y2k
 				dp->y2k = True;
 #else
+#ifdef MODE_dclock_millennium
 				dp->millennium = True;
+#else
+				dp->mayan = True;
+#endif
 #endif
 				break;
-#if defined(MODE_dclock_y2k) && defined(MODE_dclock_millennium)
+#if (defined(MODE_dclock_y2k) && defined(MODE_dclock_millennium)) ||
+(defined(MODE_dclock_millennium) && defined(MODE_dclock_mayan)) ||
+(defined(MODE_dclock_mayan) && defined(MODE_dclock_y2k))
 			case 10:
+#if (defined(MODE_dclock_y2k) && defined(MODE_dclock_millennium))
 				dp->millennium = True;
+#else
+				dp->mayan = True;
+#endif
 				break;
+#if (defined(MODE_dclock_y2k) && defined(MODE_dclock_millennium) &&
+defined(MODE_dclock_mayan)
+			case 11:
+				dp->mayan = True;
+				break;
+#endif
 #endif
 #endif
 			default:
@@ -1030,6 +1071,7 @@ init_dclock(ModeInfo * mi)
 		dp->time24 = time24;
 		dp->y2k = y2k;
 		dp->millennium = millennium;
+		dp->mayan = mayan;
 	    }
 	}
 
@@ -1066,7 +1108,7 @@ init_dclock(ModeInfo * mi)
 	dp->dy = (LRAND() & 1) ? 1 : -1;
 
 	timeNow = seconds();
-	timeLocal = timeNow - dp->tzoffset;
+	timeLocal = timeNow + dp->tzoffset;
 	if (dp->led) {
 		dp->lines = 1;
 		dp->text_descent = 0;
@@ -1207,8 +1249,16 @@ init_dclock(ModeInfo * mi)
 			dp->strptb[0] = dp->strpta[0];
 			dayhrminsec(Y2001_TIME_START - timeLocal, dp->tzoffset, dp->strnew[1]);
 			dp->strpta[1] = dp->strnew[1];
+		} else if (dp->mayan) {
+			if (MAYAN_TIME_START >= timeLocal)
+				dp->strpta[0] = (char *) MAYAN_STRING;
+			else
+				dp->strpta[0] = (char *) POST_MAYAN_STRING;
+			dp->strptb[0] = dp->strpta[0];
+			dayhrminsec(MAYAN_TIME_START - timeLocal, dp->tzoffset, dp->strnew[1]);
+			dp->strpta[1] = dp->strnew[1];
 		} else {
-			struct tm *t = localtime((const time_t *) &timeLocal);
+			struct tm *t = localtime(&timeLocal);
 
 			if (dp->time24)
 			  (void) strftime(dp->strnew[0], STRSIZE, "%H:%M:%S", t);
