@@ -1,7 +1,7 @@
 /* atlantis --- Shows moving 3D sea animals */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)atlantis.c	5.08 2003/04/09 xlockmore";
+static const char sccsid[] = "@(#)atlantis.c	5.12 2004/03/03 xlockmore";
 
 #endif
 
@@ -30,12 +30,14 @@ static const char sccsid[] = "@(#)atlantis.c	5.08 2003/04/09 xlockmore";
  * Thanks goes also to Brian Paul for making it possible and inexpensive
  * to use OpenGL at home.
  *
- * My e-mail address is lassauge@mail.dotcom.fr
+ * My e-mail address is <lassauge AT users.sourceforge.net>
  *
  * Eric Lassauge  (May-13-1998)
  *
  * REVISION HISTORY:
  *
+ * E.Lassauge - 03-Feb-2004: Integrated diffs from xscreensaver-4.15 : corrected gradient stuff
+ *
  * E.Lassauge - 09-Apr-2003: Corrected parse_image_data to finally have the texture !!!
  *
  * E.Lassauge - 21-Jan-2002: Integrated diffs from xscreensaver-4.00
@@ -46,7 +48,7 @@ static const char sccsid[] = "@(#)atlantis.c	5.08 2003/04/09 xlockmore";
  *                              were being drawn, not the front faces.
  *                            - Added a texture to simulate light from the
  *                              surface, like in the SGI version.
- * 
+ *
  * E.Lassauge - 01/03/01 : Added FPS stuff
  *
  * David A. Bagley - 98/06/17 : Add whalespeed option. Global options to
@@ -66,7 +68,6 @@ static const char sccsid[] = "@(#)atlantis.c	5.08 2003/04/09 xlockmore";
  *                        Add support for -/+ wireframe (t'was so easy to do!)
  *
  * TODO :
- *        - add a sort of background image
  *        - better handling of sizes and speeds
  *        - test standalone and module modes
  *        - purify it (!)
@@ -111,8 +112,8 @@ static const char sccsid[] = "@(#)atlantis.c	5.08 2003/04/09 xlockmore";
  * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
  */
 
-#define DEF_TEXTURE "True"
-#define DEF_GRADIENT "False"
+#define DEF_TEXTURE 	"True"
+#define DEF_GRADIENT 	"False"
 #define DEF_WHALESPEED  "250"
 
 #ifdef STANDALONE
@@ -131,10 +132,10 @@ static const char sccsid[] = "@(#)atlantis.c	5.08 2003/04/09 xlockmore";
 			 "*texture:    " DEF_TEXTURE    " \n" \
 			 "*gradient:   " DEF_GRADIENT   " \n" \
 			 "*whalespeed: " DEF_WHALESPEED " \n"
-#include "xlockmore.h"		/* from the xscreensaver distribution */
+# include "xlockmore.h"		/* from the xscreensaver distribution */
 #else /* !STANDALONE */
-#include "xlock.h"		/* from the xlockmore distribution */
-#include "visgl.h"
+# include "xlock.h"		/* from the xlockmore distribution */
+# include "visgl.h"
 #endif /* !STANDALONE */
 
 #ifdef MODE_atlantis
@@ -143,13 +144,13 @@ static const char sccsid[] = "@(#)atlantis.c	5.08 2003/04/09 xlockmore";
 #include <GL/glu.h>
 
 #ifdef HAVE_XPM
-#include "xpm-ximage.h"
+# include "xpm-ximage.h"
 
-#ifdef STANDALONE
-#include "../images/sea-texture.xpm"
-#else /* !STANDALONE */
-#include "pixmaps/sea-texture.xpm"
-#endif /* !STANDALONE */
+# ifdef STANDALONE
+#  include "../images/sea-texture.xpm"
+# else /* !STANDALONE */
+#  include "pixmaps/sea-texture.xpm"
+# endif /* !STANDALONE */
 #endif
 
 static int  whalespeed;
@@ -158,18 +159,18 @@ static int  do_gradient;
 
 static XrmOptionDescRec opts[] =
 {
-     {(char *) "-whalespeed", (char *) ".atlantis.whalespeed", XrmoptionSepArg, (caddr_t) NULL},
-     {(char *) "-texture",    (char *) ".atlantis.texture",    XrmoptionNoArg, (caddr_t)"True"},
-     {(char *) "+texture",    (char *) ".atlantis.texture",    XrmoptionNoArg, (caddr_t)"False"},
-     {(char *) "-gradient",   (char *) ".atlantis.gradient",   XrmoptionNoArg, (caddr_t)"True"},
-     {(char *) "+gradient",   (char *) ".atlantis.gradient",   XrmoptionNoArg, (caddr_t)"False"},
+	{(char *) "-whalespeed", (char *) ".atlantis.whalespeed", XrmoptionSepArg,(caddr_t)NULL},
+	{(char *) "-texture",    (char *) ".atlantis.texture",    XrmoptionNoArg, (caddr_t)"True"},
+	{(char *) "+texture",    (char *) ".atlantis.texture",    XrmoptionNoArg, (caddr_t)"False"},
+	{(char *) "-gradient",   (char *) ".atlantis.gradient",   XrmoptionNoArg, (caddr_t)"True"},
+	{(char *) "+gradient",   (char *) ".atlantis.gradient",   XrmoptionNoArg, (caddr_t)"False"},
 };
 
 static argtype vars[] =
 {
- {(caddr_t *) & whalespeed, (char *) "whalespeed", (char *) "WhaleSpeed", (char *) DEF_WHALESPEED, t_Int},
- {(caddr_t *) & do_texture, (char *) "texture",    (char *) "Texture",    (char *) DEF_TEXTURE,    t_Bool},
- {(caddr_t *) & do_gradient,(char *) "gradient",   (char *) "Gradient",   (char *) DEF_GRADIENT,   t_Bool},
+	{(void *) & whalespeed, (char *) "whalespeed", (char *) "WhaleSpeed", (char *) DEF_WHALESPEED, t_Int},
+	{(void *) & do_texture, (char *) "texture",    (char *) "Texture",    (char *) DEF_TEXTURE,    t_Bool},
+	{(void *) & do_gradient,(char *) "gradient",   (char *) "Gradient",   (char *) DEF_GRADIENT,   t_Bool},
 };
 
 static OptionStruct desc[] =
@@ -269,7 +270,7 @@ Init(ModeInfo * mi)
 	{0.4, 0.4, 0.4, 1.0};
 	static float lmodel_localviewer[] =
 	{0.0};
-	float       fblue = 0.0, fgreen;
+	float       fblue, fgreen;
 
 	glFrontFace(GL_CCW);
 
@@ -316,13 +317,13 @@ Init(ModeInfo * mi)
           {
             glDisable(GL_TEXTURE_2D);
           }
+#ifdef HAVE_XPM
         else
           {
             GLfloat s_plane[] = { 1, 0, 0, 0 };
             GLfloat t_plane[] = { 0, 0, 1, 0 };
             GLfloat scale = 0.0005;
 
-#ifdef HAVE_XPM
             glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
                          ap->texture->width, ap->texture->height, 0,
                          GL_RGBA, GL_UNSIGNED_BYTE,
@@ -353,8 +354,8 @@ Init(ModeInfo * mi)
               glScalef(scale, scale, 1);
               glMatrixMode(GL_MODELVIEW);
 	    }
-#endif
           }
+#endif
 
 	InitFishs(ap);
 
@@ -377,16 +378,9 @@ reshape_atlantis(ModeInfo * mi, int width, int height)
 	glMatrixMode(GL_MODELVIEW);
 }
 
-/* jwz -- this doesn't really work very well.
-
-   All I want to do is give the tank a gradient-filled background, instead
-   of just solid blue.  The following was my guess as to how to do this,
-   but it kills my frame rate.  I guess there's a more efficient way to do
-   this, but I don't see it...
-
-   I mean, all I want to do is dump some non-projected bytes into the color
-   buffer, then zero out the depth buffer.  That *can't* be expensive, can
-   it?
+/* Fill the background with a gradient -- thanks to
+   Phil Carrig <pod@internode.on.net> for figuring out
+   how to do this more efficiently!
  */
 static void
 clear_tank (atlantisstruct * ap)
@@ -395,91 +389,44 @@ clear_tank (atlantisstruct * ap)
 
   if (do_gradient && !ap->wire)
     {
-      static GLint gradient_list = -1;
-      static GLint gradient_tex = -1;
+      GLfloat top[3] = { 0.00, 0.40, 0.70 };
+      GLfloat bot[3] = { 0.00, 0.05, 0.18 };
 
-      if (gradient_list == -1)
+      glMatrixMode(GL_PROJECTION);
+      glPushMatrix();
+      {
+        glLoadIdentity();
+        glMatrixMode(GL_MODELVIEW);
+        glPushMatrix();
         {
-          unsigned char *pixels = 0;
-          int start = 64;
-          int end = start + 128;
-          int size = 4 * (end - start);
-          int i;
-
-          if ((pixels = (unsigned char *) malloc (size)) == NULL) {
-		do_gradient = False;
-		return;
-	  }
-          i = 0;
-          while (i < size)
-            {
-              pixels[i++] = 0;
-              pixels[i] = (start + ((i+1)>>2)) * 0.56;
-              i++;
-              pixels[i] = (start + ((i+1)>>2));
-              i++;
-              pixels[i++] = 255;
-            }
-
-          clear_gl_error();
-
-          glGenTextures(1, (unsigned int *) &gradient_tex);
-          glBindTexture(GL_TEXTURE_1D, gradient_tex);
-
-          glTexImage1D(GL_TEXTURE_1D, 0, 4,
-                       (end - start), 0,
-                       GL_RGBA, GL_UNSIGNED_BYTE, pixels);
-          if (check_gl_error ("gradient texture"))
-               return;
-          glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-          glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-
-          gradient_list = glGenLists(1);
-          glNewList(gradient_list, GL_COMPILE);
-
-          glDepthMask (False);
-          glDisable(GL_DEPTH_TEST);
-          glDisable(GL_LIGHTING);
-          glDisable(GL_TEXTURE_2D);
-          glEnable(GL_TEXTURE_1D);
-          glBindTexture(GL_TEXTURE_1D, gradient_tex);
-          glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL); 
-          glTexParameterf(GL_TEXTURE_1D, GL_TEXTURE_WRAP_S, GL_CLAMP);
-
-          glMatrixMode(GL_PROJECTION);
-          glPushMatrix();
           glLoadIdentity();
-          glRotatef(90, 0, 0, 1);
-          glTranslatef(-1, -1, 0);
-          glScalef(2, 2, 1);
 
-          glBegin(GL_QUADS);
-          glTexCoord1i(1); glVertex3i(1, 0, 0);
-          glTexCoord1i(0); glVertex3i(0, 0, 0);
-          glTexCoord1i(0); glVertex3i(0, 1, 0);
-          glTexCoord1i(1); glVertex3i(1, 1, 0);
-          glEnd();
+          /* save GL_COLOR_MATERIAL, GL_COLOR_MATERIAL_FACE, etc.
+             This stalls the pipeline, so it would be better to do this
+             with explicit enable/disable calls, but I can't figure
+             out how to undo the glEnable() and glColor() calls below!
+             Simply calling glDisable(GL_COLOR_MATERIAL) is insufficient!
+           */
+          glPushAttrib (GL_LIGHTING_BIT);
+          {
+            glEnable (GL_COLOR_MATERIAL);
 
-          glPopMatrix();
-
-          glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE); 
-          glDisable(GL_TEXTURE_1D);
-
-          glDepthMask (True);
-          glEnable(GL_DEPTH_TEST);
-          glEnable(GL_CULL_FACE);
-          glEnable(GL_LIGHTING);
-          if (do_texture)
-            glEnable(GL_TEXTURE_2D);
-
-          glEndList();
-          if (check_gl_error ("gradient list")) {
-      		do_gradient = False;
-	  	return;
-	  }
+            glShadeModel(GL_SMOOTH);
+            glBegin(GL_QUADS);
+            glColor3f (bot[0], bot[1], bot[2]); glVertex3f (-1, -1, 1);
+            glColor3f (bot[0], bot[1], bot[2]); glVertex3f ( 1, -1, 1);
+            glColor3f (top[0], top[1], top[2]); glVertex3f ( 1,  1, 1);
+            glColor3f (top[0], top[1], top[2]); glVertex3f (-1,  1, 1);
+            glEnd();
+          }
+          glPopAttrib();
         }
+        glPopMatrix();
+      }
+      glMatrixMode(GL_PROJECTION);
+      glPopMatrix();
 
-      glCallList(gradient_list);
+      glMatrixMode(GL_MODELVIEW);
     }
 }
 
@@ -564,7 +511,7 @@ init_atlantis(ModeInfo * mi)
 		if ((ap->sharks = (fishRec *) calloc(ap->num_sharks,
 						sizeof (fishRec))) == NULL) {
 			/* free everything up to now */
-			free(atlantis);
+			(void)free((void *)atlantis);
 			atlantis = (atlantisstruct *) NULL;
 			return;
 		}
@@ -617,13 +564,16 @@ draw_atlantis(ModeInfo * mi)
 
 	if (!ap->glx_context)
 		return;
-
+#ifdef WIN32
+	wglMakeCurrent(hdc, ap->glx_context);
+#else
 	glXMakeCurrent(display, window, *(ap->glx_context));
-
+#endif
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
 	AllDisplay(ap);
 	Animate(ap);
+
 	if (MI_IS_FPS(mi)) do_fps (mi);
 	glXSwapBuffers(display, window);
 }
@@ -646,13 +596,13 @@ release_atlantis(ModeInfo * mi)
 			atlantisstruct *ap = &atlantis[screen];
 
 			if (ap->sharks)
-				free(ap->sharks);
+				(void)free((void *)ap->sharks);
 			if (ap->texture)
 			{
 				XDestroyImage(ap->texture);
 			}
 		}
-		free(atlantis);
+		(void)free((void *)atlantis);
 		atlantis = (atlantisstruct *) NULL;
 	}
 	FreeAllGL(mi);
@@ -670,8 +620,11 @@ change_atlantis(ModeInfo * mi)
 
 	if (!ap->glx_context)
 		return;
-
+#ifdef WIN32
+	wglMakeCurrent(hdc, ap->glx_context);
+#else
 	glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(ap->glx_context));
+#endif
 	Init(mi);
 }
 
