@@ -1,7 +1,7 @@
 /* xpm-ximage.c --- converts XPM data to an XImage for use with OpenGL.*/
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)xpm-image.c       5.0 00/11/14 xlockmore";
+static const char sccsid[] = "@(#)xpm-ximage.c       5.0 00/11/14 xlockmore";
 
 #endif
 /*
@@ -28,8 +28,6 @@ static const char sccsid[] = "@(#)xpm-image.c       5.0 00/11/14 xlockmore";
 #include "xlock.h"              /* from the xlockmore distribution */
 #endif                          /* !STANDALONE */
 
-#include <X11/Intrinsic.h>
-
 #ifdef HAVE_XPM
 #include <X11/xpm.h>
 
