@@ -24,7 +24,7 @@ static const char sccsid[] = "@(#)life3d.c	5.27 2008/07/28 xlockmore";
  * Revision History:
  * 09-Nov-2010: cubic close packing (-neighbors 14)
  * 31-Jul-2008: 4544 3D life from paper by Carter Bays.
- * 18-Apr-2004: Added shooting gliders for S567B6
+ * 18-Apr-2004: Added shooting gliders for B6/S567
  * 13-Mar-2004: Added references
  * 28-Apr-2003: Randomized "rotation" of life form
  * 25-Jan-2003: Spawned a life3d.h
@@ -33,7 +33,7 @@ static const char sccsid[] = "@(#)life3d.c	5.27 2008/07/28 xlockmore";
  * 01-Nov-2000: Allocation checks
  * 10-May-1997: Compatible with xscreensaver
  * 18-Apr-1997: Memory leak fixed by Tom Schmidt <tschmidt@micron.com>
- * 12-Mar-1995: added LIFE_S567B6 compile-time option
+ * 12-Mar-1995: added LIFE_B6S567 compile-time option
  * 12-Feb-1995: shooting gliders added
  * 07-Dec-1994: used life.c and a DOS version of 3dlife
  * Copyright 1993 Anthony Wesley awesley@canb.auug.org.au found at
@@ -102,12 +102,12 @@ static const char sccsid[] = "@(#)life3d.c	5.27 2008/07/28 xlockmore";
 #define DEF_RULE  "G"		/* All rules with gliders */
 #else
 #define DEF_RULE  "P"		/* All rules with known patterns */
-#define DEF_RULE  "S45/B5"	/* "B5/S45" */
-#define DEF_RULE  "S567/B6"	/* "B6/S567" */
-#define DEF_RULE  "S56/B5"	/* "B5/S56" */
-#define DEF_RULE  "S678/B5"	/* "B5/S678" */
-#define DEF_RULE  "S67/B67"	/* "B67/S67" */
-/* There are no known gliders for S67/B67 */
+#define DEF_RULE  "B5/S45"
+#define DEF_RULE  "B5/S56"
+#define DEF_RULE  "B5/S678"
+#define DEF_RULE  "B6/S567"
+#define DEF_RULE  "B67/S67"
+/* There is a glider for B67/S67 but its kind of big */
 #endif
 
 #define DEF_GLIDERSEARCH  "False"
@@ -167,7 +167,7 @@ static OptionStruct desc[] =
 	{(char *) "-/+draw", (char *) "turn on/off drawing to speed search"},
 	{(char *) "-neighbors num", (char *) "cubes 6, 8, 18, 20, 26, rhombic dodecahedron 12, or truncated octahedron 14"},
 	{(char *) "-repeat num", (char *) "repeat for period to exclude in search"}, /* this is not as good as Bays' signature idea */
-	{(char *) "-rule string", (char *) "S<survial_neighborhood>/B<birth_neighborhood> parameters"},
+	{(char *) "-rule string", (char *) "B<birth_neighborhood>/S<survival_neighborhood> parameters"},
 	{(char *) "-lifefile file", (char *) "life file"},
 	{(char *) "-/+serial", (char *) "turn on/off picking of sequential patterns"},
 	{(char *) "-/+glidersearch", (char *) "search for gliders"},
@@ -201,8 +201,8 @@ ModStruct life3d_description =
 #define BASESIZE ((MAXCOLUMNS*MAXROWS*MAXSTACKS)>>6)
 #define SOUPPERCENT 30
 /* #define SOUPSIZE(s)  MAX(s/4,10) */
-/* 20S45B4 frequently has unlimited growth, which is not pretty */
-#define SOUPSIZE(s)   (lp->neighbors == 20 && lp->patterned_rule == LIFE_20S45B4) ? 4 : ((lp->neighbors == 18 && lp->patterned_rule == LIFE_18S45B4) ? 6 : 10)
+/* 20B4S45 frequently has unlimited growth, which is not pretty */
+#define SOUPSIZE(s)   (lp->neighbors == 20 && lp->patterned_rule == LIFE_20B4S45) ? 4 : ((lp->neighbors == 18 && lp->patterned_rule == LIFE_18B4S45) ? 6 : 10)
 
 #define RT_ANGLE 90
 #define HALFRT_ANGLE 45
@@ -317,32 +317,32 @@ codeToPatternedRule(int local_neighbors, paramstruct param)
 	switch (local_neighbors) {
 	case 12:
 		for (i = 0; i < LIFE_12RULES; i++)
-			if (param_12rules[i].survival == param.survival &&
-					param_12rules[i].birth == param.birth)
+			if (param_12rules[i].birth == param.birth &&
+					param_12rules[i].survival == param.survival)
 				return i;
 		return LIFE_12RULES;
 	case 14:
 		for (i = 0; i < LIFE_14RULES; i++)
-			if (param_14rules[i].survival == param.survival &&
-					param_14rules[i].birth == param.birth)
+			if (param_14rules[i].birth == param.birth &&
+					param_14rules[i].survival == param.survival)
 				return i;
 		return LIFE_14RULES;
 	case 18:
 		for (i = 0; i < LIFE_18RULES; i++)
-			if (param_18rules[i].survival == param.survival &&
-					param_18rules[i].birth == param.birth)
+			if (param_18rules[i].birth == param.birth &&
+					param_18rules[i].survival == param.survival)
 				return i;
 		return LIFE_18RULES;
 	case 20:
 		for (i = 0; i < LIFE_20RULES; i++)
-			if (param_20rules[i].survival == param.survival &&
-					param_20rules[i].birth == param.birth)
+			if (param_20rules[i].birth == param.birth &&
+					param_20rules[i].survival == param.survival)
 				return i;
 		return LIFE_20RULES;
 	case 26:
 		for (i = 0; i < LIFE_26RULES; i++)
-			if (param_26rules[i].survival == param.survival &&
-					param_26rules[i].birth == param.birth)
+			if (param_26rules[i].birth == param.birth &&
+					param_26rules[i].survival == param.survival)
 				return i;
 		return LIFE_26RULES;
 	}
@@ -358,24 +358,24 @@ copyFromPatternedRule(int local_neighbors, paramstruct * param,
 	neighborKind = invplot(local_neighbors);
 	switch (local_neighbors) {
 	case 12:
-		param->survival = param_12rules[patterned_rule].survival;
 		param->birth = param_12rules[patterned_rule].birth;
+		param->survival = param_12rules[patterned_rule].survival;
 		break;
 	case 14:
-		param->survival = param_14rules[patterned_rule].survival;
 		param->birth = param_14rules[patterned_rule].birth;
+		param->survival = param_14rules[patterned_rule].survival;
 		break;
 	case 18:
-		param->survival = param_18rules[patterned_rule].survival;
 		param->birth = param_18rules[patterned_rule].birth;
+		param->survival = param_18rules[patterned_rule].survival;
 		break;
 	case 20:
-		param->survival = param_20rules[patterned_rule].survival;
 		param->birth = param_20rules[patterned_rule].birth;
+		param->survival = param_20rules[patterned_rule].survival;
 		break;
 	case 26:
-		param->survival = param_26rules[patterned_rule].survival;
 		param->birth = param_26rules[patterned_rule].birth;
+		param->survival = param_26rules[patterned_rule].survival;
 		break;
 	}
 }
@@ -385,22 +385,21 @@ printRule(int local_neighbors, char * string, paramstruct param, Bool verbose)
 {
 	int i = 1, l;
 
-	string[0] = 'S';
+	string[0] = 'B';
 	if (verbose)
 		(void) fprintf(stdout,
-			"rule (Survival/Birth %d neighborhood): ",
+			"rule (Birth/Survival %d neighborhood): ",
 			local_neighbors);
 	for (l = 0; l <= ABS(local_neighbors) && l < 10; l++) {
-
-		if (param.survival & (1 << l)) {
+		if (param.birth & (1 << l)) {
 			(void) sprintf(&(string[i]), "%d", l);
 			i++;
 		}
 	}
-	(void) sprintf(&(string[i]), "/B");
+	(void) sprintf(&(string[i]), "/S");
 	i += 2;
 	for (l = 0; l <= ABS(local_neighbors) && l < 10; l++) {
-		if (param.birth & (1 << l)) {
+		if (param.survival & (1 << l)) {
 			(void) sprintf(&(string[i]), "%d", l);
 			i++;
 		}
@@ -408,8 +407,8 @@ printRule(int local_neighbors, char * string, paramstruct param, Bool verbose)
 	string[i] = '\0';
 	if (verbose)
 		(void) fprintf(stdout,
-			"%s\nbinary rule: Survival 0x%X, Birth 0x%X\n",
-			string, param.survival, param.birth);
+			"%s\nbinary rule: Birth 0x%X, Survival 0x%X\n",
+			string, param.birth, param.survival);
 }
 
 static void
@@ -590,53 +589,54 @@ parseRule(ModeInfo * mi, char * string)
 	life3dstruct *lp = &life3ds[MI_SCREEN(mi)];
 	int n, l = 0;
 	char serving = 0;
-	static Bool foundSurvival = False;
 	static Bool foundBirth = False;
+	static Bool foundSurvival = False;
 
-	if (foundSurvival && foundBirth)
+	if (foundBirth && foundSurvival)
 		return;
-	foundSurvival = foundBirth = False;
-	lp->input_param.survival = lp->input_param.birth = 0;
+	foundBirth = foundSurvival = False;
+	lp->input_param.birth = lp->input_param.survival = 0;
 	if (rule) {
 		n = 0;
 		while (rule[n]) {
 			if (rule[n] == 'P' || rule[n] == 'p') {
 				lp->allPatterns = True;
-				foundSurvival = foundBirth = True;
+				foundBirth = foundSurvival = True;
 				if (MI_IS_VERBOSE(mi))
 					(void) fprintf(stdout, "rule: All rules with known patterns\n");
 				return;
 			} else if (rule[n] == 'G' || rule[n] == 'g') {
 				lp->allGliders = True;
-				foundSurvival = foundBirth = True;
+				foundBirth = foundSurvival = True;
 				if (MI_IS_VERBOSE(mi))
 					(void) fprintf(stdout, "rule: All rules with known gliders\n");
 				return;
+			} else if (rule[n] == 'B' || rule[n] == 'F' ||
+					rule[n] == 'b' || rule[n] == 'f') {
+				serving = 'B';
 			} else if (rule[n] == 'S' || rule[n] == 'E' || rule[n] == 'L' ||
 					rule[n] == 's' || rule[n] == 'e' || rule[n] == 'l') {
 				serving = 'S';
-			} else if (rule[n] == 'B' || rule[n] == 'b') {
-				serving = 'B';
 			} else {
 				l = rule[n] - '0';
 				if (l >= 0 && l <= 9 && l <= ABS(lp->neighbors)) {	/* no 10..26 */
-					if (serving == 'S' || rule[n] == 's') {
-						foundSurvival = True;
-						lp->input_param.survival |= (1 << l);
-					} else if (serving == 'B' || rule[n] == 'b') {
+					if (serving == 'B' || rule[n] == 'b') {
 						foundBirth = True;
 						lp->input_param.birth |= (1 << l);
+					} else if (serving == 'S' || rule[n] == 's') {
+						foundSurvival = True;
+						lp->input_param.survival |= (1 << l);
 					}
 				}
 			}
 			n++;
 		}
 	}
-	if (!foundSurvival || !foundBirth ||
-			!(lp->input_param.survival || lp->input_param.birth)) {
+	if (!foundBirth || !foundSurvival ||
+			!(lp->input_param.birth || lp->input_param.survival)) {
 		/* Default to Bays' rules if rule does not make sense */
 		lp->allGliders = True;
-		foundSurvival = foundBirth = True;
+		foundBirth = foundSurvival = True;
 		if (MI_IS_VERBOSE(mi))
 			(void) fprintf(stdout,
 				"rule: Defaulting to all rules with known gliders\n");
@@ -1121,8 +1121,8 @@ DisplayList(life3dstruct * lp)
 #define NOSYMRAND 0
 #define ODDSYMRAND 1
 #define EVENSYMRAND 2
-#define ODDASYMRAND 3
-#define EVENASYMRAND 4
+#define ODDANTISYMRAND 3
+#define EVENANTISYMRAND 4
 #define DIAGSYMRAND 5
 
 static Bool
@@ -1276,8 +1276,8 @@ RandomSoup(ModeInfo * mi, int n, int vx, int vy, int vz)
 						SetList(2 * hc - col - 1, row, stack);
 					}
 				}
-		} else if (xrand == ODDASYMRAND) {
-			(void) sprintf(lp->nameString, "%s %srandom pattern", "x", "odd asym ");
+		} else if (xrand == ODDANTISYMRAND) {
+			(void) sprintf(lp->nameString, "%s %srandom pattern", "x", "odd antisym ");
 			for (stack = hs - vz; stack < hs + vz; ++stack)
 			for (row = hr - vy; row < hr + vy; ++row)
 			for (col = hc; col < hc + vx; ++col) {
@@ -1287,8 +1287,8 @@ RandomSoup(ModeInfo * mi, int n, int vx, int vy, int vz)
 						SetList(2 * hc - col, 2 * hr - row - 1, stack);
 					}
 				}
-		} else if (xrand == EVENASYMRAND) {
-			(void) sprintf(lp->nameString, "%s even asym pattern", "x");
+		} else if (xrand == EVENANTISYMRAND) {
+			(void) sprintf(lp->nameString, "%s even antisym pattern", "x");
 			for (stack = hs - vz; stack < hs + vz; ++stack)
 			for (row = hr - vy; row < hr + vy; ++row)
 			for (col = hc; col < hc + vx; ++col) {
@@ -1374,9 +1374,9 @@ RandomSoup(ModeInfo * mi, int n, int vx, int vy, int vz)
 					}
 				}
 		}
-	} else if (yrand == ODDASYMRAND) {
+	} else if (yrand == ODDANTISYMRAND) {
 		if (xrand == NOSYMRAND) {
-			(void) sprintf(lp->nameString, "%s odd asym pattern", "y");
+			(void) sprintf(lp->nameString, "%s odd antisym pattern", "y");
 			for (stack = hs - vz; stack < hs + vz; ++stack)
 			for (row = hr; row < hr + vy; ++row)
 			for (col = hc - vx; col < hc + vx; ++col) {
@@ -1387,9 +1387,9 @@ RandomSoup(ModeInfo * mi, int n, int vx, int vy, int vz)
 					}
 				}
 		}
-	} else if (yrand == EVENASYMRAND) {
+	} else if (yrand == EVENANTISYMRAND) {
 		if (xrand == NOSYMRAND) {
-			(void) sprintf(lp->nameString, "%s even asym pattern", "y");
+			(void) sprintf(lp->nameString, "%s even antisym pattern", "y");
 			for (stack = hs - vz; stack < hs + vz; ++stack)
 			for (row = hr; row < hr + vy; ++row)
 			for (col = hc - vx; col < hc + vx; ++col) {
@@ -1492,170 +1492,170 @@ GetPattern(ModeInfo * mi, int pattern_rule, int pattern)
 		switch (lp->neighbors) {
 		case 12:
 			switch (pattern_rule) {
-			case LIFE_12S3B3:
-				patptr = &patterns_12S3B3[pat][0];
+			case LIFE_12B3S3:
+				patptr = &patterns_12B3S3[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_12S3B3[2 * pattern][0];
+				patstrg = &patterns_12B3S3[2 * pattern][0];
 #endif
 				break;
-			case LIFE_12S456B3:
-				patptr = &patterns_12S456B3[pat][0];
+			case LIFE_12B3S456:
+				patptr = &patterns_12B3S456[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_12S456B3[2 * pattern][0];
+				patstrg = &patterns_12B3S456[2 * pattern][0];
 #endif
 				break;
 			}
 			break;
 		case 14:
 			switch (pattern_rule) {
-			case LIFE_14S34B4:
-				patptr = &patterns_14S34B4[pat][0];
+			case LIFE_14B4S34:
+				patptr = &patterns_14B4S34[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_14S34B4[2 * pattern][0];
+				patstrg = &patterns_14B4S34[2 * pattern][0];
 #endif
 				break;
-			case LIFE_14S34B46:
-				patptr = &patterns_14S34B46[pat][0];
+			case LIFE_14B46S34:
+				patptr = &patterns_14B46S34[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_14S34B46[2 * pattern][0];
+				patstrg = &patterns_14B46S34[2 * pattern][0];
 #endif
 				break;
-			case LIFE_14S56B45:
-				patptr = &patterns_14S56B45[pat][0];
+			case LIFE_14B45S56:
+				patptr = &patterns_14B45S56[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_14S56B45[2 * pattern][0];
+				patstrg = &patterns_14B45S56[2 * pattern][0];
 #endif
 				break;
 			}
 			break;
 		case 18:
 			switch (pattern_rule) {
-			case LIFE_18S45B4:
-				patptr = &patterns_18S45B4[pat][0];
+			case LIFE_18B4S45:
+				patptr = &patterns_18B4S45[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_18S45B4[2 * pattern][0];
+				patstrg = &patterns_18B4S45[2 * pattern][0];
 #endif
 				break;
 			}
 			break;
 		case 20:
 			switch (pattern_rule) {
-			case LIFE_20S45B4:
-				patptr = &patterns_20S45B4[pat][0];
+			case LIFE_20B4S45:
+				patptr = &patterns_20B4S45[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_20S45B4[2 * pattern][0];
+				patstrg = &patterns_20B4S45[2 * pattern][0];
 #endif
 				break;
 			}
 			break;
 		case 26:
 			switch (pattern_rule) {
-			case LIFE_26S23B5:
-				patptr = &patterns_26S23B5[pat][0];
+			case LIFE_26B5S23:
+				patptr = &patterns_26B5S23[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S23B5[2 * pattern][0];
+				patstrg = &patterns_26B5S23[2 * pattern][0];
 #endif
 				break;
-			case LIFE_26S25B5:
-				patptr = &patterns_26S25B5[pat][0];
+			case LIFE_26B5S25:
+				patptr = &patterns_26B5S25[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S25B5[2 * pattern][0];
+				patstrg = &patterns_26B5S25[2 * pattern][0];
 #endif
 				break;
-			case LIFE_26S27B5:
-				patptr = &patterns_26S27B5[pat][0];
+			case LIFE_26B5S27:
+				patptr = &patterns_26B5S27[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S27B5[2 * pattern][0];
+				patstrg = &patterns_26B5S27[2 * pattern][0];
 #endif
 				break;
-			case LIFE_26S35B5:
-				patptr = &patterns_26S35B5[pat][0];
+			case LIFE_26B5S35:
+				patptr = &patterns_26B5S35[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S35B5[2 * pattern][0];
+				patstrg = &patterns_26B5S35[2 * pattern][0];
 #endif
 				break;
-			case LIFE_26S36B5:
-				patptr = &patterns_26S36B5[pat][0];
+			case LIFE_26B5S36:
+				patptr = &patterns_26B5S36[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S36B5[2 * pattern][0];
+				patstrg = &patterns_26B5S36[2 * pattern][0];
 #endif
 				break;
-			case LIFE_26S37B5:
-				patptr = &patterns_26S37B5[pat][0];
+			case LIFE_26B5S37:
+				patptr = &patterns_26B5S37[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S37B5[2 * pattern][0];
+				patstrg = &patterns_26B5S37[2 * pattern][0];
 #endif
 				break;
-			case LIFE_26S38B5:
-				patptr = &patterns_26S38B5[pat][0];
+			case LIFE_26B5S38:
+				patptr = &patterns_26B5S38[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S38B5[2 * pattern][0];
+				patstrg = &patterns_26B5S38[2 * pattern][0];
 #endif
 				break;
-			case LIFE_26S4B5:
-				patptr = &patterns_26S4B5[pat][0];
+			case LIFE_26B5S4:
+				patptr = &patterns_26B5S4[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S4B5[2 * pattern][0];
+				patstrg = &patterns_26B5S4[2 * pattern][0];
 #endif
 				break;
-			case LIFE_26S45B5:
-				patptr = &patterns_26S45B5[pat][0];
+			case LIFE_26B5S45:
+				patptr = &patterns_26B5S45[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S45B5[2 * pattern][0];
+				patstrg = &patterns_26B5S45[2 * pattern][0];
 #endif
 				break;
-			case LIFE_26S47B5:
-				patptr = &patterns_26S47B5[pat][0];
+			case LIFE_26B5S47:
+				patptr = &patterns_26B5S47[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S47B5[2 * pattern][0];
+				patstrg = &patterns_26B5S47[2 * pattern][0];
 #endif
 				break;
-			case LIFE_26S56B5:
-				patptr = &patterns_26S56B5[pat][0];
+			case LIFE_26B5S56:
+				patptr = &patterns_26B5S56[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S56B5[2 * pattern][0];
+				patstrg = &patterns_26B5S56[2 * pattern][0];
 #endif
 				break;
-			case LIFE_26S567B6:
-				patptr = &patterns_26S567B6[pat][0];
+			case LIFE_26B5S58:
+				patptr = &patterns_26B5S58[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S567B6[2 * pattern][0];
+				patstrg = &patterns_26B5S58[2 * pattern][0];
 #endif
 				break;
-			case LIFE_26S57B6:
-				patptr = &patterns_26S57B6[pat][0];
+			case LIFE_26B5S678:
+				patptr = &patterns_26B5S678[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S57B6[2 * pattern][0];
+				patstrg = &patterns_26B5S678[2 * pattern][0];
 #endif
 				break;
-			case LIFE_26S57B67:
-				patptr = &patterns_26S57B67[pat][0];
+			case LIFE_26B5S8:
+				patptr = &patterns_26B5S8[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S57B67[2 * pattern][0];
+				patstrg = &patterns_26B5S8[2 * pattern][0];
 #endif
 				break;
-			case LIFE_26S58B5:
-				patptr = &patterns_26S58B5[pat][0];
+			case LIFE_26B6S567:
+				patptr = &patterns_26B6S567[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S58B5[2 * pattern][0];
+				patstrg = &patterns_26B6S567[2 * pattern][0];
 #endif
 				break;
-			case LIFE_26S678B5:
-				patptr = &patterns_26S678B5[pat][0];
+			case LIFE_26B6S57:
+				patptr = &patterns_26B6S57[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S678B5[2 * pattern][0];
+				patstrg = &patterns_26B6S57[2 * pattern][0];
 #endif
 				break;
-			case LIFE_26S8B5:
-				patptr = &patterns_26S8B5[pat][0];
+			case LIFE_26B67S57:
+				patptr = &patterns_26B67S57[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S8B5[2 * pattern][0];
+				patstrg = &patterns_26B67S57[2 * pattern][0];
 #endif
 				break;
-			case LIFE_26S67B67:
-				patptr = &patterns_26S67B67[pat][0];
+			case LIFE_26B67S67:
+				patptr = &patterns_26B67S67[pat][0];
 #ifdef LIFE_NAMES
-				patstrg = &patterns_26S67B67[2 * pattern][0];
+				patstrg = &patterns_26B67S67[2 * pattern][0];
 #endif
 				break;
 			}
@@ -2563,28 +2563,28 @@ static Bool
 orthogonal(life3dstruct * lp) /* or full diagonal */
 {
 	if (lp->neighbors == 12)
-		return (lp->patterned_rule == LIFE_12S456B3);
+		return (lp->patterned_rule == LIFE_12B3S456);
 	else if (lp->neighbors == 18)
-		return ((lp->patterned_rule == LIFE_18S45B4) &&
+		return ((lp->patterned_rule == LIFE_18B4S45) &&
 			(LRAND() & 1));
 	else if (lp->neighbors == 20)
-		return (lp->patterned_rule == LIFE_20S45B4);
+		return (lp->patterned_rule == LIFE_20B4S45);
 	else if (lp->neighbors == 26)
-		return ((lp->patterned_rule == LIFE_26S23B5) ||
-			(lp->patterned_rule == LIFE_26S25B5) ||
-			(lp->patterned_rule == LIFE_26S27B5) ||
-			(lp->patterned_rule == LIFE_26S35B5) ||
-			(lp->patterned_rule == LIFE_26S36B5) ||
-			(lp->patterned_rule == LIFE_26S37B5) ||
-			(lp->patterned_rule == LIFE_26S4B5) ||
-			(lp->patterned_rule == LIFE_26S47B5) ||
-			((lp->patterned_rule == LIFE_26S56B5) &&
+		return ((lp->patterned_rule == LIFE_26B5S23) ||
+			(lp->patterned_rule == LIFE_26B5S25) ||
+			(lp->patterned_rule == LIFE_26B5S27) ||
+			(lp->patterned_rule == LIFE_26B5S35) ||
+			(lp->patterned_rule == LIFE_26B5S36) ||
+			(lp->patterned_rule == LIFE_26B5S37) ||
+			(lp->patterned_rule == LIFE_26B5S4) ||
+			(lp->patterned_rule == LIFE_26B5S47) ||
+			((lp->patterned_rule == LIFE_26B5S56) &&
 			(LRAND() & 1)) ||
-			((lp->patterned_rule == LIFE_26S57B6) &&
+			((lp->patterned_rule == LIFE_26B6S57) &&
 			(NRAND(4) == 0)) ||
-			((lp->patterned_rule == LIFE_26S58B5) &&
+			((lp->patterned_rule == LIFE_26B5S58) &&
 			(NRAND(4) == 0)) ||
-			((lp->patterned_rule == LIFE_26S678B5) &&
+			((lp->patterned_rule == LIFE_26B5S678) &&
 			(LRAND() & 1)));
 	return False;
 }
@@ -2638,141 +2638,141 @@ static void
 populateGliders(life3dstruct * lp, int dim, int orth,
 		int hsp, int vsp, int asp, int hoff, int voff, int aoff)
 {
-	if (lp->neighbors == 12 && lp->patterned_rule == LIFE_12S3B3) {
+	if (lp->neighbors == 12 && lp->patterned_rule == LIFE_12B3S3) {
 		if (NRAND(3)) {
-			(void) populateGlider(lp, glider_12S3B3[0], dim,
+			(void) populateGlider(lp, glider_12B3S3[0], dim,
 				hsp, vsp, asp, hoff, voff, aoff);
 		} else if (NRAND(3)) {
-			(void) populateGlider(lp, glider_12S3B3[1], dim,
+			(void) populateGlider(lp, glider_12B3S3[1], dim,
 				hsp, vsp, asp, hoff, voff, aoff);
 		} else { /* lower probability of big glider */
-			(void) populateGlider(lp, glider_12S3B3[2], dim,
+			(void) populateGlider(lp, glider_12B3S3[2], dim,
 				hsp, vsp, asp, hoff, voff, aoff);
 		}
-	} else if (lp->neighbors == 12 && lp->patterned_rule == LIFE_12S456B3) {
-		(void) populateGlider(lp, glider_12S456B3[0], dim,
+	} else if (lp->neighbors == 12 && lp->patterned_rule == LIFE_12B3S456) {
+		(void) populateGlider(lp, glider_12B3S456[0], dim,
 			hsp, vsp, asp, hoff, voff, aoff);
-	} else if (lp->neighbors == 14 && lp->patterned_rule == LIFE_14S34B4) {
-		(void) populateGlider(lp, glider_14S34B4[0], dim,
+	} else if (lp->neighbors == 14 && lp->patterned_rule == LIFE_14B4S34) {
+		(void) populateGlider(lp, glider_14B4S34[0], dim,
 			hsp, vsp, asp, hoff, voff, aoff);
-	} else if (lp->neighbors == 14 && lp->patterned_rule == LIFE_14S34B46) {
-		(void) populateGlider(lp, glider_14S34B46[0], dim,
+	} else if (lp->neighbors == 14 && lp->patterned_rule == LIFE_14B46S34) {
+		(void) populateGlider(lp, glider_14B46S34[0], dim,
 			hsp, vsp, asp, hoff, voff, aoff);
-	} else if (lp->neighbors == 18 && lp->patterned_rule == LIFE_18S45B4) {
+	} else if (lp->neighbors == 18 && lp->patterned_rule == LIFE_18B4S45) {
 		if (orth) {
 			if (LRAND() & 1) {
-				(void) populateGlider(lp, glider_18S45B4[0], dim,
+				(void) populateGlider(lp, glider_18B4S45[0], dim,
 					hsp, vsp, asp, hoff, voff, aoff);
 			} else {
-				(void) populateGlider(lp, glider_18S45B4[1], dim,
+				(void) populateGlider(lp, glider_18B4S45[1], dim,
 					hsp, vsp, asp, hoff, voff, aoff);
 			}
 		} else {
-			(void) populateGlider(lp, glider_18S45B4[2], dim,
+			(void) populateGlider(lp, glider_18B4S45[2], dim,
 				hsp, vsp, asp, hoff, voff, aoff);
 		}
-	} else if (lp->neighbors == 20 && lp->patterned_rule == LIFE_20S45B4) {
+	} else if (lp->neighbors == 20 && lp->patterned_rule == LIFE_20B4S45) {
 		if (LRAND() & 1) {
-			(void) populateGlider(lp, glider_20S45B4[0], dim,
+			(void) populateGlider(lp, glider_20B4S45[0], dim,
 				hsp, vsp, asp, hoff, voff, aoff);
 		} else {
-			(void) populateGlider(lp, glider_20S45B4[1], dim,
+			(void) populateGlider(lp, glider_20B4S45[1], dim,
 				hsp, vsp, asp, hoff, voff, aoff);
 		}
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26S23B5) {
-		(void) populateGlider(lp, glider_26S23B5[0], dim,
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B5S23) {
+		(void) populateGlider(lp, glider_26B5S23[0], dim,
 			hsp, vsp, asp, hoff, voff, aoff);
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26S25B5) {
-		(void) populateGlider(lp, glider_26S25B5[0], dim,
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B5S25) {
+		(void) populateGlider(lp, glider_26B5S25[0], dim,
 			hsp, vsp, asp, hoff, voff, aoff);
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26S27B5) {
-		(void) populateGlider(lp, glider_26S27B5[0], dim,
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B5S27) {
+		(void) populateGlider(lp, glider_26B5S27[0], dim,
 			hsp, vsp, asp, hoff, voff, aoff);
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26S35B5) {
-		(void) populateGlider(lp, glider_26S35B5[0], dim,
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B5S35) {
+		(void) populateGlider(lp, glider_26B5S35[0], dim,
 			hsp, vsp, asp, hoff, voff, aoff);
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26S36B5) {
-		(void) populateGlider(lp, glider_26S36B5[0], dim,
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B5S36) {
+		(void) populateGlider(lp, glider_26B5S36[0], dim,
 			hsp, vsp, asp, hoff, voff, aoff);
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26S37B5) {
-		(void) populateGlider(lp, glider_26S37B5[0], dim,
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B5S37) {
+		(void) populateGlider(lp, glider_26B5S37[0], dim,
 			hsp, vsp, asp, hoff, voff, aoff);
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26S38B5) {
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B5S38) {
 		if (LRAND() & 1) {
-			(void) populateGlider(lp, glider_26S38B5[0], dim,
+			(void) populateGlider(lp, glider_26B5S38[0], dim,
 				hsp, vsp, asp, hoff, voff, aoff);
 		} else {
-			(void) populateGlider(lp, glider_26S38B5[1], dim,
+			(void) populateGlider(lp, glider_26B5S38[1], dim,
 				hsp, vsp, asp, hoff, voff, aoff);
 		}
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26S4B5) {
-		(void) populateGlider(lp, glider_26S4B5[0], dim,
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B5S4) {
+		(void) populateGlider(lp, glider_26B5S4[0], dim,
 			hsp, vsp, asp, hoff, voff, aoff);
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26S45B5) {
-		(void) populateGlider(lp, glider_26S45B5[0], dim,
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B5S45) {
+		(void) populateGlider(lp, glider_26B5S45[0], dim,
 			hsp, vsp, asp, hoff, voff, aoff);
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26S47B5) {
-		(void) populateGlider(lp, glider_26S47B5[0], dim,
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B5S47) {
+		(void) populateGlider(lp, glider_26B5S47[0], dim,
 			hsp, vsp, asp, hoff, voff, aoff);
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26S56B5) {
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B5S56) {
 		if (orth) {
 			if (LRAND() & 1) {
-				(void) populateGlider(lp, glider_26S56B5[0], dim,
+				(void) populateGlider(lp, glider_26B5S56[0], dim,
 					hsp, vsp, asp, hoff, voff, aoff);
 			} else {
-				(void) populateGlider(lp, glider_26S56B5[1], dim,
+				(void) populateGlider(lp, glider_26B5S56[1], dim,
 					hsp, vsp, asp, hoff, voff, aoff);
 			}
 		} else {
-			(void) populateGlider(lp, glider_26S56B5[2], dim,
+			(void) populateGlider(lp, glider_26B5S56[2], dim,
 				hsp, vsp, asp, hoff, voff, aoff);
 		}
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26S567B6) {
-		(void) populateGlider(lp, glider_26S567B6[0], dim,
-			hsp, vsp, asp, hoff, voff, aoff);
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26S57B6) {
-		if (orth) {
-			if (LRAND() & 1) {
-				(void) populateGlider(lp, glider_26S57B6[1], dim,
-					hsp, vsp, asp, hoff, voff, aoff);
-			} else {
-				(void) populateGlider(lp, glider_26S57B6[2], dim,
-					hsp, vsp, asp, hoff, voff, aoff);
-			}
-		} else {
-			(void) populateGlider(lp, glider_26S57B6[0], dim,
-				hsp, vsp, asp, hoff, voff, aoff);
-		}
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26S57B67) {
-		(void) populateGlider(lp, glider_26S57B67[0], dim,
-			hsp, vsp, asp, hoff, voff, aoff);
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26S58B5) {
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B5S58) {
 		if (orth)
-			(void) populateGlider(lp, glider_26S58B5[4], dim,
+			(void) populateGlider(lp, glider_26B5S58[4], dim,
 				hsp, vsp, asp, hoff, voff, aoff);
 		else if (NRAND(3)) {
 			if (LRAND() & 1)
-				(void) populateGlider(lp, glider_26S58B5[0], dim,
+				(void) populateGlider(lp, glider_26B5S58[0], dim,
 					hsp, vsp, asp, hoff, voff, aoff);
 			else if (LRAND() & 1)
-				(void) populateGlider(lp, glider_26S58B5[1], dim,
+				(void) populateGlider(lp, glider_26B5S58[1], dim,
 					hsp, vsp, asp, hoff, voff, aoff);
 			else
-				(void) populateGlider(lp, glider_26S58B5[2], dim,
+				(void) populateGlider(lp, glider_26B5S58[2], dim,
 					hsp, vsp, asp, hoff, voff, aoff);
 		} else
-			(void) populateGlider(lp, glider_26S58B5[3], dim,
+			(void) populateGlider(lp, glider_26B5S58[3], dim,
 				hsp, vsp, asp, hoff, voff, aoff);
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26S678B5) {
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B5S678) {
 		if (orth) {
-			(void) populateGlider(lp, glider_26S678B5[1], dim,
+			(void) populateGlider(lp, glider_26B5S678[1], dim,
 				hsp, vsp, asp, hoff, voff, aoff);
 		} else {
-			(void) populateGlider(lp, glider_26S678B5[0], dim,
+			(void) populateGlider(lp, glider_26B5S678[0], dim,
 				hsp, vsp, asp, hoff, voff, aoff);
 		}
-	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26S8B5) {
-		(void) populateGlider(lp, glider_26S8B5[0], dim,
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B5S8) {
+		(void) populateGlider(lp, glider_26B5S8[0], dim,
+			hsp, vsp, asp, hoff, voff, aoff);
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B6S567) {
+		(void) populateGlider(lp, glider_26B6S567[0], dim,
+			hsp, vsp, asp, hoff, voff, aoff);
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B6S57) {
+		if (orth) {
+			if (LRAND() & 1) {
+				(void) populateGlider(lp, glider_26B6S57[1], dim,
+					hsp, vsp, asp, hoff, voff, aoff);
+			} else {
+				(void) populateGlider(lp, glider_26B6S57[2], dim,
+					hsp, vsp, asp, hoff, voff, aoff);
+			}
+		} else {
+			(void) populateGlider(lp, glider_26B6S57[0], dim,
+				hsp, vsp, asp, hoff, voff, aoff);
+		}
+	} else if (lp->neighbors == 26 && lp->patterned_rule == LIFE_26B67S57) {
+		(void) populateGlider(lp, glider_26B67S57[0], dim,
 			hsp, vsp, asp, hoff, voff, aoff);
 	}
 }
@@ -3027,8 +3027,8 @@ init_life3d(ModeInfo * mi)
 		printRule(lp->neighbors, lp->ruleString, lp->param,
 			MI_IS_VERBOSE(mi));
 	} else {
-		lp->param.survival = lp->input_param.survival;
 		lp->param.birth = lp->input_param.birth;
+		lp->param.survival = lp->input_param.survival;
 		lp->patterned_rule = codeToPatternedRule(lp->neighbors, lp->param);
 		printRule(lp->neighbors, lp->ruleString, lp->param,
 			MI_IS_VERBOSE(mi));
@@ -3050,6 +3050,7 @@ init_life3d(ModeInfo * mi)
 	}
 	lp->colors[0] = MI_BLACK_PIXEL(mi);
 	if (MI_NPIXELS(mi) > 2) {
+#ifdef DISTRIB_COLOR
 		i = NRAND(3);
 
 		lp->colors[i + 1] = MI_PIXEL(mi,
@@ -3060,6 +3061,15 @@ init_life3d(ModeInfo * mi)
 		lp->colors[(i + 2) % 3 + 1] = MI_PIXEL(mi,
 			NRAND(MI_NPIXELS(mi) / COLORBASE) +
 			2 * MI_NPIXELS(mi) / COLORBASE);
+#else
+		int j = NRAND(MI_NPIXELS(mi));
+		i = NRAND(3);
+		lp->colors[i + 1] =  MI_PIXEL(mi, j);
+		lp->colors[(i + 1) % 3 + 1] =
+			MI_PIXEL(mi, (j + 2) % MI_NPIXELS(mi));
+		lp->colors[(i + 2) % 3 + 1] =
+			MI_PIXEL(mi, (j + 4) % MI_NPIXELS(mi));
+#endif
 	} else {
 		/*lp->colors[1] = lp->colors[2] = lp->colors[3] =
 			MI_WHITE_PIXEL(mi);*/
@@ -3082,7 +3092,7 @@ init_life3d(ModeInfo * mi)
 	/*lp->tablesMade = 0; */
 
 	if (MI_IS_FULLRANDOM(mi)) {
-		lp->wireframe = (NRAND(8) == 0);
+		lp->wireframe = (NRAND(16) == 0);
 	} else {
 		lp->wireframe = MI_IS_WIREFRAME(mi);
 	}
