@@ -2,7 +2,7 @@
 /* voters --- Dewdney's Voting Simulation */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)voters.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)voters.c	5.24 2007/01/18 xlockmore";
 
 #endif
 
@@ -22,6 +22,7 @@ static const char sccsid[] = "@(#)voters.c	5.00 2000/11/01 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
+ * 18-Jan-2007: Added vertical option.
  * 01-Nov-2000: Allocation checks
  * 10-Jun-1997: Coded from A.K. Dewdney's "The Armchair Universe, Computer
  *              Recreations from the Pages of Scientific American Magazine"
@@ -54,22 +55,27 @@ static const char sccsid[] = "@(#)voters.c	5.00 2000/11/01 xlockmore";
  * neighbors of 0 randomizes it between 3, 4, 6, 8, 9, and 12.
  */
 #define DEF_NEIGHBORS  "0"      /* choose random value */
+#define DEF_VERTICAL "False"
 
 static int  neighbors;
+static Bool vertical;
 
 static XrmOptionDescRec opts[] =
 {
-        {(char *) "-neighbors", (char *) ".voters.neighbors", XrmoptionSepArg, (caddr_t) NULL}
+        {(char *) "-neighbors", (char *) ".voters.neighbors", XrmoptionSepArg, (caddr_t) NULL},
+	{(char *) "-vertical", (char *) ".voters.vertical", XrmoptionNoArg, (caddr_t) "on"},
+        {(char *) "+vertical", (char *) ".voters.vertical", XrmoptionNoArg, (caddr_t) "off"}
 };
 
 static argtype vars[] =
 {
-        {(void *) & neighbors, (char *) "neighbors", (char *) "Neighbors", (char *) DEF_NEIGHBORS, t_Int
-}
+        {(void *) & neighbors, (char *) "neighbors", (char *) "Neighbors", (char *) DEF_NEIGHBORS, t_Int},
+	{(void *) & vertical, (char *) "vertical", (char *) "Vertical", (char *) DEF_VERTICAL, t_Bool}
 };
 static OptionStruct desc[] =
 {
-        {(char *) "-neighbors num", (char *) "squares 4 or 8, hexagons 6, triangles 3, 9 or 12"}
+        {(char *) "-neighbors num", (char *) "squares 4 or 8, hexagons 6, triangles 3, 9 or 12"},
+	{(char *) "-/+vertical", (char *) "change orientation for hexagons and triangles"}
 };
 
 ModeSpecOpt voters_opts =
@@ -91,9 +97,13 @@ ModStruct   voters_description =
  * case anyone wants to know....  Please, no fascist "improvements".  :)
  */
 
-#include "bitmaps/sickle.xbm"
-#include "bitmaps/donkey.xbm"
 #include "bitmaps/elephant.xbm"
+#ifdef COMMIE
+#include "bitmaps/sickle.xbm"
+#else
+#include "bitmaps/green.xbm"
+#endif
+#include "bitmaps/donkey.xbm"
 
 #define MINPARTIES 2
 #define BITMAPS 3
@@ -104,9 +114,9 @@ ModStruct   voters_description =
 
 static XImage logo[BITMAPS] =
 {
-      {0, 0, 0, XYBitmap, (char *) sickle_bits, LSBFirst, 8, LSBFirst, 8, 1},
-      {0, 0, 0, XYBitmap, (char *) donkey_bits, LSBFirst, 8, LSBFirst, 8, 1},
     {0, 0, 0, XYBitmap, (char *) elephant_bits, LSBFirst, 8, LSBFirst, 8, 1},
+    {0, 0, 0, XYBitmap, (char *) green_bits, LSBFirst, 8, LSBFirst, 8, 1},
+    {0, 0, 0, XYBitmap, (char *) donkey_bits, LSBFirst, 8, LSBFirst, 8, 1},
 };
 
 /* Voter data */
@@ -123,8 +133,8 @@ typedef struct _CellList {
 } CellList;
 
 typedef struct {
-	Bool        painted;
-	int         party;	/* Currently working on donkey, elephant, or sickle? */
+	Bool        painted, vertical;
+	int         party;	/* Currently working on donkey, elephant, or green? */
 	int         xs, ys;	/* Size of party icon */
 	int         xb, yb;	/* Bitmap offset for party icon */
 	int         nparties;	/* 2 parties or 3 */
@@ -136,10 +146,13 @@ typedef struct {
 	int         width, height;
 	CellList   *last, *first;
 	char       *arr;
-	int         neighbors;
+	int         neighbors, polygon;
 	int         busyLoop;
-	XPoint      hexagonList[6];
-	XPoint      triangleList[2][3];
+	union {
+		XPoint      hexagon[6];
+		XPoint      triangle[2][3];
+		XPoint      pentagon[4][5];
+	} shape;
 } voterstruct;
 
 static char plots[NEIGHBORKINDS] =
@@ -151,64 +164,102 @@ static voterstruct *voters = (voterstruct *) NULL;
 static int  icon_width, icon_height;
 
 static void
-drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
-	 Bool firstChange)
+drawCell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
+		Bool firstChange)
 {
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
 	GC          gc = MI_GC(mi);
 	voterstruct *vp = &voters[MI_SCREEN(mi)];
 	unsigned long colour = (MI_NPIXELS(mi) > 2) ?
-	MI_PIXEL(mi, color) : MI_WHITE_PIXEL(mi);
+		MI_PIXEL(mi, color) : MI_WHITE_PIXEL(mi);
 
 	XSetForeground(display, gc, colour);
 	if (vp->neighbors == 6) {
-		int         ccol = 2 * col + !(row & 1), crow = 2 * row;
+		int ccol = 2 * col + !(row & 1), crow = 2 * row;
 
-		vp->hexagonList[0].x = vp->xb + ccol * vp->xs;
-		vp->hexagonList[0].y = vp->yb + crow * vp->ys;
+		if (vp->vertical) {
+			vp->shape.hexagon[0].x = vp->xb + ccol * vp->xs;
+			vp->shape.hexagon[0].y = vp->yb + crow * vp->ys;
+		} else {
+			vp->shape.hexagon[0].y = vp->xb + ccol * vp->xs;
+			vp->shape.hexagon[0].x = vp->yb + crow * vp->ys;
+		}
 		if (vp->xs == 1 && vp->ys == 1)
 			XDrawPoint(display, window, gc,
-			   vp->hexagonList[0].x, vp->hexagonList[0].y);
+				vp->shape.hexagon[0].x,
+				vp->shape.hexagon[0].y);
 		else if (bitmap == BITMAPS - 1)
 			XFillPolygon(display, window, gc,
-			      vp->hexagonList, 6, Convex, CoordModePrevious);
+				vp->shape.hexagon, 6,
+				Convex, CoordModePrevious);
 		else {
+			int ix = 0, iy = 0, sx, sy;
+
 			if (firstChange) {
-				XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
+				XSetForeground(display, gc,
+					MI_BLACK_PIXEL(mi));
 				XFillPolygon(display, window, gc,
-					     vp->hexagonList, 6, Convex, CoordModePrevious);
+					vp->shape.hexagon, 6,
+					Convex, CoordModePrevious);
 				XSetForeground(display, gc, colour);
 			}
+			if (vp->vertical) {
+				vp->shape.hexagon[0].x -= vp->xs;
+				vp->shape.hexagon[0].y += vp->ys / 4;
+				sx = 2 * vp->xs - 6;
+				sy = 2 * vp->ys - 2;
+				if (vp->xs <= 6 || vp->ys <= 2) {
+					ix = 3;
+					iy = 1;
+				} else
+					ix = 5;
+			} else {
+				vp->shape.hexagon[0].y -= vp->xs;
+				vp->shape.hexagon[0].x += vp->ys / 4;
+				sy = 2 * vp->xs - 6;
+				sx = 2 * vp->ys - 2;
+				if (vp->xs <= 6 || vp->ys <= 2) {
+					iy = 3;
+					ix = 1;
+				} else
+					iy = 5;
+			}
 			if (vp->xs <= 6 || vp->ys <= 2)
 				XFillRectangle(display, window, gc,
-				       vp->hexagonList[0].x - 3 * vp->xs / 4,
-					       vp->hexagonList[0].y + vp->ys / 4, vp->xs, vp->ys);
+					vp->shape.hexagon[0].x + ix, 
+					vp->shape.hexagon[0].y + iy,
+					vp->xs, vp->ys);
 			else
 				XFillArc(display, window, gc,
-				     vp->xb + vp->xs * ccol - 3 * vp->xs / 4,
-					 vp->yb + vp->ys * crow + vp->ys / 4,
-					 2 * vp->xs - 6, 2 * vp->ys - 2,
-					 0, 23040);
+					vp->shape.hexagon[0].x + ix,
+					vp->shape.hexagon[0].y + iy,
+					sy, sy,
+					0, 23040);
 		}
 	} else if (vp->neighbors == 4 || vp->neighbors == 8) {
 		if (vp->pixelmode) {
 			if (bitmap == BITMAPS - 1 || (vp->xs <= 2 || vp->ys <= 2))
 				XFillRectangle(display, window, gc,
-				vp->xb + vp->xs * col, vp->yb + vp->ys * row,
-				vp->xs - (vp->xs > 3), vp->ys - (vp->ys > 3));
+					vp->xb + vp->xs * col,
+					vp->yb + vp->ys * row,
+					vp->xs - (vp->xs > 3),
+					vp->ys - (vp->ys > 3));
 			else {
 				if (firstChange) {
-					XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
+					XSetForeground(display, gc,
+						MI_BLACK_PIXEL(mi));
 					XFillRectangle(display, window, gc,
-						       vp->xb + vp->xs * col, vp->yb + vp->ys * row,
-						       vp->xs, vp->ys);
+						vp->xb + vp->xs * col,
+						vp->yb + vp->ys * row,
+						vp->xs, vp->ys);
 					XSetForeground(display, gc, colour);
 				}
 				XFillArc(display, window, gc,
-					 vp->xb + vp->xs * col, vp->yb + vp->ys * row,
-					 vp->xs - 1, vp->ys - 1,
-					 0, 23040);
+					vp->xb + vp->xs * col,
+					vp->yb + vp->ys * row,
+					vp->xs - 1, vp->ys - 1,
+					0, 23040);
 			}
 		} else
 			(void) XPutImage(display, window, gc,
@@ -216,34 +267,63 @@ drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
 				vp->xb + vp->xs * col, vp->yb + vp->ys * row,
 					 icon_width, icon_height);
 	} else {		/* TRI */
-		int         orient = (col + row) % 2;	/* O left 1 right */
+		int orient = (col + row) % 2;	/* O left 1 right */
+		Bool small = (vp->xs <= 3 || vp->ys <= 3);
+		int ix = 0, iy = 0;
 
-		vp->triangleList[orient][0].x = vp->xb + col * vp->xs;
-		vp->triangleList[orient][0].y = vp->yb + row * vp->ys;
-		if (vp->xs <= 3 || vp->ys <= 3)
-			XDrawPoint(display, window, gc,
-			 ((orient) ? -1 : 1) + vp->triangleList[orient][0].x,
-				       vp->triangleList[orient][0].y);
-		else {
-			if (orient)
-				vp->triangleList[orient][0].x += (vp->xs / 2 - 1);
+		if (vp->vertical) {
+			vp->shape.triangle[orient][0].x = vp->xb + col * vp->xs;
+			vp->shape.triangle[orient][0].y = vp->yb + row * vp->ys;
+			if (small)
+				vp->shape.triangle[orient][0].x +=
+					((orient) ? -1 : 1);
 			else
-				vp->triangleList[orient][0].x -= (vp->xs / 2 - 1);
+				vp->shape.triangle[orient][0].x +=
+					(vp->xs / 2  - 1) * ((orient) ? 1 : -1);
+		} else {
+			vp->shape.triangle[orient][0].y = vp->xb + col * vp->xs;
+			vp->shape.triangle[orient][0].x = vp->yb + row * vp->ys;
+			if (small)
+				vp->shape.triangle[orient][0].y +=
+					((orient) ? -1 : 1);
+			else
+				vp->shape.triangle[orient][0].y +=
+					(vp->xs / 2  - 1) * ((orient) ? 1 : -1);
+		}
+		if (small)
+			XDrawPoint(display, window, gc,
+				vp->shape.triangle[orient][0].x,
+				vp->shape.triangle[orient][0].y);
+		else {
 			if (bitmap == BITMAPS - 1)
 				XFillPolygon(display, window, gc,
-					     vp->triangleList[orient], 3, Convex, CoordModePrevious);
+					vp->shape.triangle[orient], 3,
+					Convex, CoordModePrevious);
 			else {
 				if (firstChange) {
-					XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
+					XSetForeground(display, gc,
+						MI_BLACK_PIXEL(mi));
 					XFillPolygon(display, window, gc,
-						     vp->triangleList[orient], 3, Convex, CoordModePrevious);
+						vp->shape.triangle[orient], 3,
+						Convex, CoordModePrevious);
 					XSetForeground(display, gc, colour);
 				}
+				if (vp->vertical) {
+					vp->shape.triangle[orient][0].x += -4 * vp->xs / 5 +
+						((orient) ? vp->xs / 3 : 3 * vp->xs / 5);
+					vp->shape.triangle[orient][0].y += -vp->ys / 2 + 1;
+					ix = ((orient) ? -vp->xs / 2 : vp->xs / 2);
+				} else {
+					vp->shape.triangle[orient][0].y += -4 * vp->xs / 5 +
+						((orient) ? vp->xs / 3 : 3 * vp->xs / 5);
+					vp->shape.triangle[orient][0].x += -vp->ys / 2 + 1;
+					iy = ((orient) ? -vp->xs / 2 : vp->xs / 2);
+				}
 				XFillArc(display, window, gc,
-				     vp->xb + vp->xs * col - 4 * vp->xs / 5 +
-				    ((orient) ? vp->xs / 3 : 3 * vp->xs / 5),
-					 vp->yb + vp->ys * row - vp->ys / 2 + 1, vp->ys - 3, vp->ys - 3,
-					 0, 23040);
+					vp->shape.triangle[orient][0].x + ix,
+					vp->shape.triangle[orient][0].y + iy,
+					vp->ys - 3, vp->ys - 3,
+					0, 23040);
 			}
 		}
 	}
@@ -302,225 +382,366 @@ flush_list(voterstruct * vp)
 	}
 }
 
-static char
-neighbors_opinion(voterstruct * vp, int col, int row, int dir)
+
+static int
+positionOfNeighbor(voterstruct * vp, int n, int col, int row)
 {
-	if (vp->neighbors == 6) {
+	int dir = n * (360 / vp->neighbors);
+
+	if (vp->polygon == 4 || vp->polygon == 6) {
 		switch (dir) {
-			case 0:
+		case 0:
+			col = (col + 1 == vp->ncols) ? 0 : col + 1;
+			break;
+		case 45:
+			col = (col + 1 == vp->ncols) ? 0 : col + 1;
+			row = (!row) ? vp->nrows - 1 : row - 1;
+			break;
+		case 60:
+			if (!(row & 1))
 				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+			row = (!row) ? vp->nrows - 1 : row - 1;
+			break;
+		case 90:
+			row = (!row) ? vp->nrows - 1 : row - 1;
+			break;
+		case 120:
+			if (row & 1)
+				col = (!col) ? vp->ncols - 1 : col - 1;
+			row = (!row) ? vp->nrows - 1 : row - 1;
+			break;
+		case 135:
+			col = (!col) ? vp->ncols - 1 : col - 1;
+			row = (!row) ? vp->nrows - 1 : row - 1;
+			break;
+		case 180:
+			col = (!col) ? vp->ncols - 1 : col - 1;
+			break;
+		case 225:
+			col = (!col) ? vp->ncols - 1 : col - 1;
+			row = (row + 1 == vp->nrows) ? 0 : row + 1;
+			break;
+		case 240:
+			if (row & 1)
+				col = (!col) ? vp->ncols - 1 : col - 1;
+			row = (row + 1 == vp->nrows) ? 0 : row + 1;
+			break;
+		case 270:
+			row = (row + 1 == vp->nrows) ? 0 : row + 1;
+			break;
+		case 300:
+			if (!(row & 1))
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+			row = (row + 1 == vp->nrows) ? 0 : row + 1;
+			break;
+		case 315:
+			col = (col + 1 == vp->ncols) ? 0 : col + 1;
+			row = (row + 1 == vp->nrows) ? 0 : row + 1;
+			break;
+		default:
+			(void) fprintf(stderr, "wrong direction %d\n", dir);
+		}
+	} else if (vp->polygon == 3) {
+		if ((col + row) & 1) {	/* right */
+			switch (dir) {
+			case 0:
+				col = (!col) ? vp->ncols - 1 : col - 1;
+				break;
+			case 30:
+			case 40:
+				col = (!col) ? vp->ncols - 1 : col - 1;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
 				break;
 			case 60:
-				if (!(row & 1))
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-				row = (!row) ? vp->nrows - 1 : row - 1;
+				col = (!col) ? vp->ncols - 1 : col - 1;
+				if (row + 1 == vp->nrows)
+					row = 1;
+				else if (row + 2 == vp->nrows)
+					row = 0;
+				else
+					row = row + 2;
+				break;
+			case 80:
+			case 90:
+				if (row + 1 == vp->nrows)
+					row = 1;
+				else if (row + 2 == vp->nrows)
+					row = 0;
+				else
+					row = row + 2;
 				break;
 			case 120:
-				if (row & 1)
-					col = (!col) ? vp->ncols - 1 : col - 1;
-				row = (!row) ? vp->nrows - 1 : row - 1;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 150:
+			case 160:
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
 				break;
 			case 180:
-				col = (!col) ? vp->ncols - 1 : col - 1;
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+				break;
+			case 200:
+			case 210:
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+				row = (!row) ? vp->nrows - 1 : row - 1;
 				break;
 			case 240:
-				if (row & 1)
-					col = (!col) ? vp->ncols - 1 : col - 1;
-				row = (row + 1 == vp->nrows) ? 0 : row + 1;
-				break;
-			case 300:
-				if (!(row & 1))
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-				row = (row + 1 == vp->nrows) ? 0 : row + 1;
-				break;
-			default:
-				(void) fprintf(stderr, "wrong direction %d\n", dir);
-		}
-	} else if (vp->neighbors == 4 || vp->neighbors == 8) {
-		switch (dir) {
-			case 0:
-				col = (col + 1 == vp->ncols) ? 0 : col + 1;
-				break;
-			case 45:
-				col = (col + 1 == vp->ncols) ? 0 : col + 1;
 				row = (!row) ? vp->nrows - 1 : row - 1;
 				break;
-			case 90:
-				row = (!row) ? vp->nrows - 1 : row - 1;
-				break;
-			case 135:
-				col = (!col) ? vp->ncols - 1 : col - 1;
-				row = (!row) ? vp->nrows - 1 : row - 1;
-				break;
-			case 180:
-				col = (!col) ? vp->ncols - 1 : col - 1;
-				break;
-			case 225:
-				col = (!col) ? vp->ncols - 1 : col - 1;
-				row = (row + 1 == vp->nrows) ? 0 : row + 1;
-				break;
 			case 270:
-				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+			case 280:
+				if (!row)
+					row = vp->nrows - 2;
+				else if (!(row - 1))
+					row = vp->nrows - 1;
+				else
+					row = row - 2;
 				break;
-			case 315:
-				col = (col + 1 == vp->ncols) ? 0 : col + 1;
-				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+			case 300:
+				col = (!col) ? vp->ncols - 1 : col - 1;
+				if (!row)
+					row = vp->nrows - 2;
+				else if (!(row - 1))
+					row = vp->nrows - 1;
+				else
+					row = row - 2;
+				break;
+			case 320:
+			case 330:
+				col = (!col) ? vp->ncols - 1 : col - 1;
+				row = (!row) ? vp->nrows - 1 : row - 1;
 				break;
 			default:
-				(void) fprintf(stderr, "wrong direction %d\n", dir);
-		}
-	} else {		/* TRI */
-		if ((col + row) % 2) {	/* right */
-			switch (dir) {
-				case 0:
-					col = (!col) ? vp->ncols - 1 : col - 1;
-					break;
-				case 30:
-				case 40:
-					col = (!col) ? vp->ncols - 1 : col - 1;
-					row = (row + 1 == vp->nrows) ? 0 : row + 1;
-					break;
-				case 60:
-					col = (!col) ? vp->ncols - 1 : col - 1;
-					if (row + 1 == vp->nrows)
-						row = 1;
-					else if (row + 2 == vp->nrows)
-						row = 0;
-					else
-						row = row + 2;
-					break;
-				case 80:
-				case 90:
-					if (row + 1 == vp->nrows)
-						row = 1;
-					else if (row + 2 == vp->nrows)
-						row = 0;
-					else
-						row = row + 2;
-					break;
-				case 120:
-					row = (row + 1 == vp->nrows) ? 0 : row + 1;
-					break;
-				case 150:
-				case 160:
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-					row = (row + 1 == vp->nrows) ? 0 : row + 1;
-					break;
-				case 180:
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-					break;
-				case 200:
-				case 210:
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-					row = (!row) ? vp->nrows - 1 : row - 1;
-					break;
-				case 240:
-					row = (!row) ? vp->nrows - 1 : row - 1;
-					break;
-				case 270:
-				case 280:
-					if (!row)
-						row = vp->nrows - 2;
-					else if (!(row - 1))
-						row = vp->nrows - 1;
-					else
-						row = row - 2;
-
-					break;
-				case 300:
-					col = (!col) ? vp->ncols - 1 : col - 1;
-					if (!row)
-						row = vp->nrows - 2;
-					else if (!(row - 1))
-						row = vp->nrows - 1;
-					else
-						row = row - 2;
-					break;
-				case 320:
-				case 330:
-					col = (!col) ? vp->ncols - 1 : col - 1;
-					row = (!row) ? vp->nrows - 1 : row - 1;
-					break;
-				default:
-					(void) fprintf(stderr, "wrong direction %d\n", dir);
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
 			}
 		} else {	/* left */
 			switch (dir) {
-				case 0:
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-					break;
-				case 30:
-				case 40:
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-					row = (!row) ? vp->nrows - 1 : row - 1;
-					break;
-				case 60:
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-					if (!row)
-						row = vp->nrows - 2;
-					else if (row == 1)
-						row = vp->nrows - 1;
-					else
-						row = row - 2;
-					break;
-				case 80:
-				case 90:
-					if (!row)
-						row = vp->nrows - 2;
-					else if (row == 1)
-						row = vp->nrows - 1;
-					else
-						row = row - 2;
-					break;
-				case 120:
-					row = (!row) ? vp->nrows - 1 : row - 1;
-					break;
-				case 150:
-				case 160:
-					col = (!col) ? vp->ncols - 1 : col - 1;
-					row = (!row) ? vp->nrows - 1 : row - 1;
-					break;
-				case 180:
-					col = (!col) ? vp->ncols - 1 : col - 1;
-					break;
-				case 200:
-				case 210:
-					col = (!col) ? vp->ncols - 1 : col - 1;
-					row = (row + 1 == vp->nrows) ? 0 : row + 1;
-					break;
-				case 240:
-					row = (row + 1 == vp->nrows) ? 0 : row + 1;
-					break;
-				case 270:
-				case 280:
-					if (row + 1 == vp->nrows)
-						row = 1;
-					else if (row + 2 == vp->nrows)
-						row = 0;
-					else
-						row = row + 2;
-					break;
-				case 300:
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-					if (row + 1 == vp->nrows)
-						row = 1;
-					else if (row + 2 == vp->nrows)
-						row = 0;
-					else
-						row = row + 2;
-					break;
-				case 320:
-				case 330:
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-					row = (row + 1 == vp->nrows) ? 0 : row + 1;
-					break;
-				default:
-					(void) fprintf(stderr, "wrong direction %d\n", dir);
+			case 0:
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+				break;
+			case 30:
+			case 40:
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+				row = (!row) ? vp->nrows - 1 : row - 1;
+				break;
+			case 60:
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+				if (!row)
+					row = vp->nrows - 2;
+				else if (row == 1)
+					row = vp->nrows - 1;
+				else
+					row = row - 2;
+				break;
+			case 80:
+			case 90:
+				if (!row)
+					row = vp->nrows - 2;
+				else if (row == 1)
+					row = vp->nrows - 1;
+				else
+					row = row - 2;
+				break;
+			case 120:
+				row = (!row) ? vp->nrows - 1 : row - 1;
+				break;
+			case 150:
+			case 160:
+				col = (!col) ? vp->ncols - 1 : col - 1;
+				row = (!row) ? vp->nrows - 1 : row - 1;
+				break;
+			case 180:
+				col = (!col) ? vp->ncols - 1 : col - 1;
+				break;
+			case 200:
+			case 210:
+				col = (!col) ? vp->ncols - 1 : col - 1;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 240:
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 270:
+			case 280:
+				if (row + 1 == vp->nrows)
+					row = 1;
+				else if (row + 2 == vp->nrows)
+					row = 0;
+				else
+					row = row + 2;
+				break;
+			case 300:
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+				if (row + 1 == vp->nrows)
+					row = 1;
+				else if (row + 2 == vp->nrows)
+					row = 0;
+				else
+					row = row + 2;
+				break;
+			case 320:
+			case 330:
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
 			}
 		}
+#if 0
+	} else {
+		int orient = ((row & 1) * 2 + col) % 4;
+		switch (orient) { /* up, down, left, right */
+		case 0:
+			switch (dir) {
+			case 0:
+				col++;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col + 2 >= vp->ncols) ? 0 : col + 2;
+				break;
+			case 102: /* 7 corner */
+				col = (col + 3 >= vp->ncols) ? 1 : col + 3;
+				row = (row == 0) ? vp->nrows - 1 : row - 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col++;
+				row = (row == 0) ? vp->nrows - 1 : row - 1;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				row = (row == 0) ? vp->nrows - 1 : row - 1;
+				break;
+			case 255: /* 7 */
+				col = (col == 0) ? vp->ncols - 1 : col - 1;
+				row = (row == 0) ? vp->nrows - 1 : row - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				col = (col == 0) ? vp->ncols - 1 : col - 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 1:
+			switch (dir) {
+			case 0:
+				col--;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col == 1) ? vp->ncols - 1 : col - 2;
+				break;
+			case 102: /* 7 */
+				col = (col == 1) ? vp->ncols - 2 : col - 3;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col--;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 255: /* 7 */
+				col = (col + 1 >= vp->ncols) ? 0 : col + 1;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				col = (col + 1 >= vp->ncols) ? 0 : col + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 2:
+			switch (dir) {
+			case 0:
+				col = (col + 1 >= vp->ncols) ? 0 : col + 1;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				row = (row == 0) ? vp->nrows - 1 : row - 1;
+				col++;
+				break;
+			case 102: /* 7 */
+				col = (col == 0) ? vp->ncols - 1 : col - 1;
+				row = (row == 0) ? vp->nrows - 1 : row - 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col = (col == 0) ? vp->ncols - 2 : col - 2;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				col = (col == 0) ? vp->ncols - 1 : col - 1;
+				break;
+			case 255: /* 7 */
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				col = (col == 0) ? vp->ncols - 1 : col - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 3:
+			switch (dir) {
+			case 0:
+				col--;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col == 0) ? vp->ncols - 1 : col - 1;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 102: /* 7 */
+				col = (col + 1 >= vp->ncols) ? 0 : col + 1;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col = (col + 2 >= vp->ncols) ? 1 : col + 2;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				col = (col + 1 >= vp->ncols) ? 0 : col + 1;
+				break;
+			case 255: /* 7 */
+				col = (col + 1 >= vp->ncols) ? 0 : col + 1;
+				row = (row == 0) ? vp->nrows - 1 : row - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				row = (row == 0) ? vp->nrows - 1 : row - 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		default:
+			(void) fprintf(stderr, "wrong orient %d\n",
+				orient);
+		}
+#endif
 	}
-
-	return vp->arr[row * vp->ncols + col];
+	return (row * vp->ncols + col);
 }
 
 static void
@@ -539,7 +760,7 @@ advanceColors(ModeInfo * mi, int col, int row)
 				curr->info.age--;
 			else if (curr->info.age < 0)
 				curr->info.age++;
-			drawcell(mi, curr->info.col, curr->info.row,
+			drawCell(mi, curr->info.col, curr->info.row,
 				 (MI_NPIXELS(mi) + curr->info.age / FACTOR +
 				  (MI_NPIXELS(mi) * curr->info.kind / BITMAPS)) % MI_NPIXELS(mi),
 				 curr->info.kind, False);
@@ -571,7 +792,7 @@ refresh_voters(ModeInfo * mi)
 			for (col = 0; col < vp->ncols; col++) {
 				colrow = col + row * vp->ncols;
 				/* Draw all old, will get corrected soon if wrong... */
-				drawcell(mi, col, row,
+				drawCell(mi, col, row,
 					 (unsigned long) (MI_NPIXELS(mi) * vp->arr[colrow] / BITMAPS),
 					 vp->arr[colrow], False);
 			}
@@ -625,7 +846,11 @@ init_voters(ModeInfo * mi)
 		}
 	} else			/* Exterminate all free thinking individuals */
 		flush_list(vp);
-
+	if (MI_IS_FULLRANDOM(mi)) {
+		vp->vertical = (Bool) (LRAND() & 1);
+	} else {
+		vp->vertical = vertical;
+	}
 	vp->width = MI_WIDTH(mi);
 	vp->height = MI_HEIGHT(mi);
 
@@ -646,8 +871,13 @@ init_voters(ModeInfo * mi)
 	}
 
 	if (vp->neighbors == 6) {
-		int         nccols, ncrows, sides;
+		int nccols, ncrows, sides;
 
+		vp->polygon = 6;
+		if (!vp->vertical) {
+			vp->height = MI_WIDTH(mi);
+			vp->width = MI_HEIGHT(mi);
+		}
 		if (vp->width < 8)
 			vp->width = 8;
 		if (vp->height < 8)
@@ -670,13 +900,25 @@ init_voters(ModeInfo * mi)
 		vp->ncols = nccols / 2;
 		vp->nrows = 2 * (ncrows / 4);
 		vp->xb = (vp->width - vp->xs * nccols) / 2 + vp->xs / 2;
-		vp->yb = (vp->height - vp->ys * (ncrows / 2) * 2) / 2 + vp->ys - 2;
+		vp->yb = (vp->height - vp->ys * (ncrows / 2) * 2) / 2 +
+			vp->ys - 2;
 		for (sides = 0; sides < 6; sides++) {
-			vp->hexagonList[sides].x = (vp->xs - 1) * hexagonUnit[sides].x;
-			vp->hexagonList[sides].y =
-				((vp->ys - 1) * hexagonUnit[sides].y / 2) * 4 / 3;
+			if (vp->vertical) {
+				vp->shape.hexagon[sides].x =
+					(vp->xs - 1) * hexagonUnit[sides].x;
+				vp->shape.hexagon[sides].y =
+					((vp->ys - 1) * hexagonUnit[sides].y /
+					2) * 4 / 3;
+			} else {
+				vp->shape.hexagon[sides].y =
+					(vp->xs - 1) * hexagonUnit[sides].x;
+				vp->shape.hexagon[sides].x =
+					((vp->ys - 1) * hexagonUnit[sides].y /
+					2) * 4 / 3;
+			}
 		}
 	} else if (vp->neighbors == 4 || vp->neighbors == 8) {
+		vp->polygon = 4;
 		if (vp->width < 2)
 			vp->width = 2;
 		if (vp->height < 2)
@@ -710,8 +952,13 @@ init_voters(ModeInfo * mi)
 		vp->xb = (vp->width - vp->xs * vp->ncols) / 2;
 		vp->yb = (vp->height - vp->ys * vp->nrows) / 2;
 	} else {		/* TRI */
-		int         orient, sides;
+		int orient, sides;
 
+		vp->polygon = 3;
+		if (!vp->vertical) {
+			vp->height = MI_WIDTH(mi);
+			vp->width = MI_HEIGHT(mi);
+		}
 		if (vp->width < 2)
 			vp->width = 2;
 		if (vp->height < 2)
@@ -735,10 +982,17 @@ init_voters(ModeInfo * mi)
 		vp->yb = (vp->height - vp->ys * vp->nrows) / 2 + vp->ys / 2;
 		for (orient = 0; orient < 2; orient++) {
 			for (sides = 0; sides < 3; sides++) {
-				vp->triangleList[orient][sides].x =
-					(vp->xs - 2) * triangleUnit[orient][sides].x;
-				vp->triangleList[orient][sides].y =
-					(vp->ys - 2) * triangleUnit[orient][sides].y;
+				if (vp->vertical) {
+					vp->shape.triangle[orient][sides].x =
+						(vp->xs - 2) * triangleUnit[orient][sides].x;
+					vp->shape.triangle[orient][sides].y =
+						(vp->ys - 2) * triangleUnit[orient][sides].y;
+				} else {
+					vp->shape.triangle[orient][sides].y =
+						(vp->xs - 2) * triangleUnit[orient][sides].x;
+					vp->shape.triangle[orient][sides].x =
+						(vp->ys - 2) * triangleUnit[orient][sides].y;
+				}
 			}
 		}
 	}
@@ -768,9 +1022,12 @@ init_voters(ModeInfo * mi)
 	for (row = 0; row < vp->nrows; row++)
 		for (col = 0; col < vp->ncols; col++) {
 			colrow = col + row * vp->ncols;
-			i = NRAND(vp->nparties) + (vp->nparties == 2);
+			if (vp->nparties == 2)
+				i = (NRAND(vp->nparties) + 2) % BITMAPS;
+			else
+				i = NRAND(vp->nparties);
 			vp->arr[colrow] = (char) i;
-			drawcell(mi, col, row, (unsigned long) (MI_NPIXELS(mi) * i / BITMAPS),
+			drawCell(mi, col, row, (unsigned long) (MI_NPIXELS(mi) * i / BITMAPS),
 				 i, False);
 			vp->number_in_party[i]++;
 		}
@@ -805,12 +1062,12 @@ draw_voters(ModeInfo * mi)
 			init_voters(mi);	/* Create a more interesting planet */
 		}
 	spineless_dude = NRAND(vp->npositions);
-	neighbor_direction = NRAND(vp->neighbors) * 360 / vp->neighbors;
+	neighbor_direction = NRAND(vp->neighbors);
 	spineless_col = spineless_dude % vp->ncols;
 	spineless_row = spineless_dude / vp->ncols;
 	old_opinion = vp->arr[spineless_dude];
-	new_opinion = neighbors_opinion(vp, spineless_col, spineless_row,
-					neighbor_direction);
+	/* neighbors opinion */
+	new_opinion = vp->arr[positionOfNeighbor(vp, neighbor_direction, spineless_col, spineless_row)];
 	if (old_opinion != new_opinion) {
 		vp->number_in_party[old_opinion]--;
 		vp->number_in_party[new_opinion]++;
@@ -831,7 +1088,7 @@ draw_voters(ModeInfo * mi)
 				return;
 			}
 		}
-		drawcell(mi, spineless_col, spineless_row,
+		drawCell(mi, spineless_col, spineless_row,
 			 (MI_NPIXELS(mi) + info.age / FACTOR +
 		  (MI_NPIXELS(mi) * new_opinion / BITMAPS)) % MI_NPIXELS(mi),
 			 new_opinion, True);
